'''
This module defines classes needed to manipulate c++ types from pythran.
'''

from inspect import isclass


class ordered_set(object):

    def __init__(self, elements=None):
        self.values = list()
        self.unique_values = set()
        if elements is not None:
            for elt in elements:
                self.append(elt)

    def append(self, value):
        if value not in self.unique_values:
            self.values.append(value)
            self.unique_values.add(value)

    def __iter__(self):
        return iter(self.values)

    def __len__(self):
        return len(self.values)

    def __getitem__(self, index):
        return self.values[index]


class TypeBuilder(object):
    '''
    >>> builder = TypeBuilder()

    >>> builder.NamedType('long long')
    long long

    >>> l_ty = builder.NamedType('long')
    >>> i_ty = builder.NamedType('int')
    >>> f_ty = builder.NamedType('float')

    >>> l_ty + builder.NamedType('long')
    long
    >>> builder.NamedType('long') + builder.NamedType('char')
    typename __combined<long,char>::type

    >>> builder.ArgumentType(4)
    typename std::remove_cv<typename std::remove_reference<argument_type4>::\
type>::type

    >>> builder.Assignable(builder.NamedType("long"))
    typename pythonic::assignable<long>::type

    >>> builder.Returnable(builder.NamedType("long"))
    typename pythonic::returnable<long>::type

    >>> builder.Lazy(builder.NamedType("long"))
    typename pythonic::lazy<long>::type

    >>> builder.DeclType("toto")
    typename std::remove_cv<\
typename std::remove_reference<decltype(toto)>::type>::type

    >>> builder.IteratorOfType(builder.NamedType('some'))
    typename some::iterator
    >>> builder.IteratorOfType(builder.NamedType('typename some::stuff'))
    typename some::stuff::iterator

    >>> builder.IteratorContentType(builder.NamedType('str'))
    typename std::remove_cv<typename std::iterator_traits<\
typename std::remove_reference<str>::type::iterator>::value_type>::type

    >>> builder.GetAttr(builder.NamedType('complex'), 'real')
    decltype(pythonic::builtins::getattr(\
pythonic::types::attr::REAL{}, std::declval<complex>()))

    >>> builder.ReturnType(builder.NamedType('math::cos'), [f_ty])
    decltype(std::declval<math::cos>()(std::declval<float>()))

    >>> t = builder.TupleType([i_ty, builder.NamedType('str')])
    >>> builder.ElementType(1, t)
    typename std::tuple_element<1,typename std::remove_reference<\
decltype(pythonic::types::make_tuple(std::declval<int>(), \
std::declval<str>()))>::type>::type


    >>> builder.ListType(builder.NamedType('int'))
    pythonic::types::list<typename std::remove_reference<int>::type>

    >>> builder.SetType(builder.NamedType('int'))
    pythonic::types::set<int>

    >>> builder.TupleType([i_ty, builder.NamedType('bool')])
    decltype(pythonic::types::make_tuple(std::declval<int>(), \
std::declval<bool>()))

    >>> builder.DictType(builder.NamedType('int'), builder.NamedType('float'))
    pythonic::types::dict<int,float>

    >>> builder.ContainerType(builder.NamedType('int'))
    container<typename std::remove_reference<int>::type>

    >>> builder.IndexableType(builder.NamedType('int'))
    indexable<int>

    >>> op = lambda x,y: x + '+' + y
    >>> builder.ExpressionType(op, [l_ty, i_ty])
    decltype(std::declval<long>()+std::declval<int>())
    '''

    def __init__(builder):
        builder._instances = dict()

        class Type(object):
            """
            A generic type object to be sub-classed

            The keyword arguments are used to built the internal representation
            one attribute per key with the associated value
            """

            def __new__(cls, *args, **kwargs):
                # no memoization for PType
                if cls.__name__ == 'PType':
                    return super(Type, cls).__new__(cls)

                key = cls,
                for v in args + tuple(v for k, v in sorted(kwargs.items())):
                    if isinstance(v, list):
                        v = tuple(v)
                    key += v,

                if key not in builder._instances:
                    builder._instances[key] = super(Type, cls).__new__(cls)

                return builder._instances[key]

            def __init__(self, **kwargs):
                for k, v in kwargs.items():
                    if isinstance(v, list):
                        v = tuple(v)
                    setattr(self, k, v)

            def iscombined(self):
                return False

            def __add__(self, other):
                if self is other:
                    return self
                return CombinedTypes(self, other)

            def __repr__(self):
                return self.generate(str)

        class NamedType(Type):
            """
            A generic type object, to hold scalar types and such
            """

            def __init__(self, srepr):
                super(NamedType, self).__init__(srepr=srepr)

            def generate(self, _):
                return self.srepr

        class PType(Type):
            """
            A generic parametric type
            """

            prefix = "__ptype{0}"
            count = 0

            def __init__(self, fun, ptype):
                super(PType, self).__init__(fun=fun,
                                            type=ptype,
                                            name=PType.prefix.format(
                                                PType.count))
                PType.count += 1

            def generate(self, ctx):
                return ctx(self.type)

            def instanciate(self, caller, arguments):
                if self.fun is caller:
                    return builder.UnknownType
                else:
                    return InstantiatedType(self.fun, self.name, arguments)

        class LType(Type):
            def __init__(self, base, node):
                super(LType, self).__init__(node=node)
                self.isrec = False
                self.orig = base
                self.final_type = base

            def generate(self, ctx):
                if self.isrec:
                    return ctx(self.orig)
                else:
                    self.isrec = True
                    return ctx(self.final_type)

        class InstantiatedType(Type):
            """
            A type instantiated from a parametric type
            """
            def __init__(self, fun, name, arguments):
                super(InstantiatedType, self).__init__(fun=fun,
                                                       name=name,
                                                       arguments=arguments)

            def generate(self, ctx):
                if self.arguments:
                    args = ", ".join(ctx(arg) for arg in self.arguments)
                    template_params = "<{0}>".format(args)
                else:
                    template_params = ""

                return "typename {0}::type{1}::{2}".format(self.fun.name,
                                                           template_params,
                                                           self.name)

        class CombinedTypes(Type):
            """
            type resulting from the combination of other types

            """

            def __init__(self, *types):
                super(CombinedTypes, self).__init__(types=types)

            def iscombined(self):
                return True

            def __add__(self, other):
                worklist = list(self.types)
                visited = set()
                while worklist:
                    item = worklist.pop()
                    if item is other:
                        return self
                    if item in visited:
                        continue
                    visited.add(item)
                    if isinstance(item, CombinedTypes):
                        worklist.extend(item.types)
                return Type.__add__(self, other)

            def __radd__(self, other):
                return self.__add__(other)

            def generate(self, ctx):
                # Degenerated trees may lead to very deep call stacks.
                # In that case try hard to recover by cutting the tree
                import sys
                current_recursion_limit = sys.getrecursionlimit()
                stypes = []
                for t in self.types:
                    try:
                        stypes.append(ctx(t))
                    except RecursionError:
                        sys.setrecursionlimit(current_recursion_limit * 2)
                        break

                if not stypes:
                    sys.setrecursionlimit(current_recursion_limit)
                    raise RecursionError
                elif len(stypes) == 1:
                    sys.setrecursionlimit(current_recursion_limit)
                    return stypes[0]
                else:
                    stmp = 'typename __combined<{}>::type'.format(
                        ','.join(stypes))
                    sys.setrecursionlimit(current_recursion_limit)
                    return stmp

        class ArgumentType(Type):
            """
            A type to hold function arguments
            """
            def __init__(self, num):
                super(ArgumentType, self).__init__(num=num)

            def generate(self, _):
                argtype = "argument_type{0}".format(self.num)
                noref = "typename std::remove_reference<{0}>::type".format(
                    argtype)
                return "typename std::remove_cv<{0}>::type".format(noref)

        class DependentType(Type):
            """
            A class to be sub-classed by any type that depends on another type
            """
            def __init__(self, of):
                assert of is not None
                super(DependentType, self).__init__(of=of)

            def iscombined(self):
                return self.of.iscombined()

        class Assignable(DependentType):
            """
            A type which can be assigned

            It is used to make the difference between
            * transient types (e.g. generated from expression template)
            * assignable types (typically type of a variable)
            """

            def generate(self, ctx):
                return 'typename pythonic::assignable<{0}>::type'.format(
                    ctx(self.of))

        class AssignableNoEscape(DependentType):
            """
            Similar to Assignable, but it doesn't escape it's declaration scope
            """

            def generate(self, ctx):
                return 'typename pythonic::assignable_noescape<{0}>::type'.format(
                    ctx(self.of))

        class Returnable(DependentType):
            """
            A type which can be returned

            It is used to make the difference between
            * returned types (that cannot hold a reference to avoid dangling
                              reference)
            * assignable types (local to a function)

            """

            def generate(self, ctx):
                return 'typename pythonic::returnable<{0}>::type'.format(
                    ctx(self.of))

        class Lazy(DependentType):
            """
            A type which can be a reference

            It is used to make a lazy evaluation of numpy expressions

            """

            def generate(self, ctx):
                return 'typename pythonic::lazy<{}>::type'.format(ctx(self.of))

        class DeclType(NamedType):
            """
            Gather the type of a variable
            """

            def generate(self, _):
                return ('typename std::remove_cv<'
                        'typename std::remove_reference<'
                        'decltype({0})>::type>::type'.format(self.srepr))

        class IteratorOfType(DependentType):
            '''
            Type of an Iterator of a container
            '''
            def generate(self, ctx):
                container_type = ctx(self.of)
                if container_type.startswith('typename'):
                    return container_type + '::iterator'
                else:
                    return 'typename ' + container_type + '::iterator'

        class IteratorContentType(DependentType):
            '''
            Type of an iterator over the content of a container
            '''

            def generate(self, ctx):
                iterator_value_type = ctx(self.of)
                return 'typename std::remove_cv<{0}>::type'.format(
                    'typename std::iterator_traits<{0}>::value_type'.format(
                        'typename std::remove_reference<{0}>::type::iterator'
                        .format(iterator_value_type)
                        )
                    )

        class GetAttr(Type):
            '''
            Type of a named attribute
            '''
            def __init__(self, param, attr):
                super(GetAttr, self).__init__(param=param, attr=attr)

            def generate(self, ctx):
                return ('decltype(pythonic::builtins::getattr({}{{}}, {}))'
                        .format('pythonic::types::attr::' + self.attr.upper(),
                                'std::declval<' + ctx(self.param) + '>()'))

        class ReturnType(Type):
            '''
            Return type of a call with arguments
            '''
            def __init__(self, ftype, args):
                super(ReturnType, self).__init__(ftype=ftype, args=args)

            def generate(self, ctx):
                # the return type of a constructor is obvious
                cg = 'std::declval<{0}>()'.format(ctx(self.ftype))
                args = ("std::declval<{0}>()".format(ctx(arg))
                        for arg in self.args)
                return 'decltype({0}({1}))'.format(cg, ", ".join(args))

        class ElementType(Type):
            '''
            Type of the ith element of a tuple or container
            '''

            def __init__(self, index, of):
                super(ElementType, self).__init__(of=of, index=index)

            def iscombined(self):
                return self.of.iscombined()

            def generate(self, ctx):
                return 'typename std::tuple_element<{0},{1}>::type'.format(
                    self.index,
                    'typename std::remove_reference<{0}>::type'.format(
                        ctx(self.of)
                        )
                    )

        class ListType(DependentType):
            '''
            Type holding a list of stuff of the same type
            '''

            def generate(self, ctx):
                return 'pythonic::types::list<{}>'.format(
                    'typename std::remove_reference<{0}>::type'.format(
                        ctx(self.of)))

        class SetType(DependentType):
            '''
            Type holding a set of stuff of the same type
            '''

            def generate(self, ctx):
                return 'pythonic::types::set<{0}>'.format(ctx(self.of))

        class TupleType(Type):
            '''
            Type holding a tuple of stuffs of various types
            '''
            def __init__(self, ofs):
                super(TupleType, self).__init__(ofs=ofs)

            def iscombined(self):
                return any(of.iscombined() for of in self.ofs)

            def generate(self, ctx):
                elts = (ctx(of) for of in self.ofs)
                telts = ('std::declval<{0}>()'.format(elt) for elt in elts)
                return 'decltype(pythonic::types::make_tuple({0}))'.format(
                    ", ".join(telts))

        class DictType(Type):
            '''
            Type holding a dict of stuff of the same key and value type
            '''

            def __init__(self, of_key, of_val):
                super(DictType, self).__init__(of_key=of_key, of_val=of_val)

            def iscombined(self):
                return any((of.iscombined()
                            for of in (self.of_key, self.of_val)))

            def generate(self, ctx):
                return 'pythonic::types::dict<{},{}>'.format(ctx(self.of_key),
                                                             ctx(self.of_val))

        class ContainerType(DependentType):
            '''
            Type of any container of stuff of the same type
            '''

            def generate(self, ctx):
                return ('container<typename std::remove_reference<{0}>::type>'
                        .format(ctx(self.of)))

        class IndexableType(DependentType):
            '''
            Type of any container indexed by the same type
            '''

            def generate(self, ctx):
                return 'indexable<{0}>'.format(ctx(self.of))

        class IndexableContainerType(Type):
            '''
            Type of any container of stuff of the same type,
            indexable by another type
            '''
            def __init__(self, of_key, of_val):
                super(IndexableContainerType, self).__init__(of_key=of_key,
                                                             of_val=of_val)

            def iscombined(self):
                return any((of.iscombined()
                            for of in (self.of_key, self.of_val)))

            def generate(self, ctx):
                return ('indexable_container<'
                        '{0}, typename std::remove_reference<{1}>::type'
                        '>'
                        .format(ctx(self.of_key), ctx(self.of_val)))

        class ExpressionType(Type):

            """
            Result type of an operator call.
            """

            def __init__(self, op, exprs):
                super(ExpressionType, self).__init__(op=op, exprs=exprs)

            def iscombined(self):
                return any(expr.iscombined() for expr in self.exprs)

            def generate(self, ctx):
                gexprs = ["std::declval<{0}>()".format(ctx(expr))
                          for expr in self.exprs]
                return 'decltype({0})'.format(self.op(*gexprs))

        builder.UnknownType = Type()

        for objname, obj in locals().items():
            if isclass(obj):
                setattr(builder, objname, obj)



def omp_get_num_threads():
    import omp
    nthreads = 0
    nthreads_lib = -1

    if 'omp parallel':
        if 'omp critical':
            nthreads += 1
        if 'omp single':
            nthreads_lib = omp.get_num_threads()

    return nthreads == nthreads_lib



#runas solve()
#pythran export solve()
def solve():
    '''
    Find the first four consecutive integers to have four distinct prime factors
    '''

    prime_list = [2, 3, 5, 7, 11, 13, 17, 19, 23]   # Ensure that this is initialised with at least 1 prime
    prime_dict = dict.fromkeys(prime_list, 1)

    def _isprime(n):
        ''' Raw check to see if n is prime. Assumes that prime_list is already populated '''
        isprime = n >= 2 and 1 or 0
        for prime in prime_list:                    # Check for factors with all primes
            if prime * prime > n: break             # ... up to sqrt(n)
            if not n % prime:
                isprime = 0
            break
        if isprime: prime_dict[n] = 1               # Maintain a dictionary for fast lookup
        return isprime
    def _refresh(x):
        ''' Refreshes primes upto x '''
        lastn = prime_list[-1]
        while lastn <= x:                           # Keep working until we've got up to x
            lastn = lastn + 1                       # Check the next number
            if _isprime(lastn):
                prime_list.append(lastn)            # Maintain a list for sequential access

    def primes_factors(n):
        ''' Returns a prime factors of n as a list '''
        _refresh(n)
        x, xp, f = 0, prime_list[0], []
        while xp <= n:
            if not n % xp:
                f.append(xp)
                n = n / xp
            else:
                x = x + 1
                xp = prime_list[x]
        return f

    def distinct_factors(n): return len(dict.fromkeys(primes_factors(n)).keys())

    factors = [0, 1, distinct_factors(2), distinct_factors(3)]
    while True:
        if factors[-4::] == [4,4,4,4]: break
        else: factors.append(distinct_factors(len(factors)))

    return len(factors)-4




#pythran export clip(complex128[], float64), clip(complex128[::], float64)
#runas import numpy as np ; a = np.arange(2, dtype=complex); clip(a, .5), clip(a[2::4], .5)

import numpy as np

def limit1 (x, epsilon=1e-6):
    if abs(x) < epsilon:
        return 0
    else:
        return x / abs(x)

def clip (z, _max):
    out = np.empty (z.shape, dtype=z.dtype)
    for i in range (len(z)):
        if abs(z[i]) > _max:
            out[i] = limit1 (z[i]) * _max
        else:
            out[i] = z[i]

    return out



#runas calc(60,100)
#bench calc(120,200)
#pythran export calc(int, int)
def update(u):
    dx = 0.1
    dy = 0.1
    dx2 = dx*dx
    dy2 = dy*dy
    nx, ny = len(u), len(u[0])
    for i in range(1,nx-1):
        for j in range(1, ny-1):
            u[i][j] = ((u[i+1][ j] + u[i-1][ j]) * dy2 +
                    (u[i][ j+1] + u[i][ j-1]) * dx2) / (2*(dx2+dy2))

def calc(N, Niter=100):
    u = [ [0]*N for _ in range(N)]
    u[0] = [1] * N
    for i in range(Niter):
        update(u)
    return u



import unittest
from pythran.tests import TestEnv
from pythran.typing import List


@TestEnv.module
class TestBisect(TestEnv):
    def test_bisect_left0(self):
        self.run_test("def bisect_left0(l,a): from bisect import bisect_left ; return bisect_left(l,a)", [0,1,2,3],2, bisect_left0=[List[int],int])

    def test_bisect_left1(self):
        self.run_test("def bisect_left1(l,a): from bisect import bisect_left ; return bisect_left(l,a,1)", [0,1,2,3],2, bisect_left1=[List[int],int])

    def test_bisect_left2(self):
        self.run_test("def bisect_left2(l,a): from bisect import bisect_left ; return bisect_left(l,a)", [1,1,1,1],1, bisect_left2=[List[int],int])

    def test_bisect_left3(self):
        self.run_test("def bisect_left3(l,a): from bisect import bisect_left ; return bisect_left(l,a,1,2)", [0,1,1,3],2, bisect_left3=[List[int],int])

    def test_bisect_left4(self):
        self.run_test("def bisect_left4(l,a): from bisect import bisect_left ; return bisect_left(l,a)", [1,1,1,1],2, bisect_left4=[List[int],int])

    def test_bisect_right0(self):
        self.run_test("def bisect_right0(l,a): from bisect import bisect_right ; return bisect_right(l,a)", [0,1,2,3],2, bisect_right0=[List[int],int])

    def test_bisect_right1(self):
        self.run_test("def bisect_right1(l,a): from bisect import bisect_right ; return bisect_right(l,a,1)", [0,1,2,3],2, bisect_right1=[List[int],int])

    def test_bisect_right2(self):
        self.run_test("def bisect_right2(l,a): from bisect import bisect_right ; return bisect_right(l,a)", [1,1,1,1],1, bisect_right2=[List[int],int])

    def test_bisect_right3(self):
        self.run_test("def bisect_right3(l,a): from bisect import bisect_right ; return bisect_right(l,a,1,2)", [0,1,1,3],2, bisect_right3=[List[int],int])

    def test_bisect_right4(self):
        self.run_test("def bisect_right4(l,a): from bisect import bisect_right ; return bisect_right(l,a)", [1,1,1,1],2, bisect_right4=[List[int],int])

    def test_bisect_raise0(self):
        with self.assertRaises(ValueError):
            self.run_test("def bisect_raise0(l): from bisect import bisect ; return bisect(l,1, -1)", [0,1,2,3],bisect_raise0=[List[int]])




# from https://github.com/numba/numba/blob/master/examples/fbcorr.py
#pythran export fbcorr(float list list list list, float list list list list)
#pythran export fbcorr(int list list list list, int list list list list)
#runas imgs = [ [ [ [ i+j+k for i in range(3) ] for j in range(16) ] for j in range(16) ] for k in range(16) ]; filters = [ [ [ [ i+2*j-k for i in range(3) ] for j in range(5) ] for j in range(5) ] for k in range(6) ] ; fbcorr(imgs, filters)
#bench imgs = [ [ [ [ i+j+k for i in range(11) ] for j in range(16) ] for j in range(16) ] for k in range(16) ]; filters = [ [ [ [ i+2*j-k for i in range(11) ] for j in range(5) ] for j in range(5) ] for k in range(6) ] ; fbcorr(imgs, filters)

def fbcorr(imgs, filters):
    n_imgs, n_rows, n_cols, n_channels = (len(imgs), len(imgs[0]), len(imgs[0][0]), len(imgs[0][0][0]))
    n_filters, height, width, n_ch2 = (len(filters), len(filters[0]), len(filters[0][0]), len(filters[0][0][0]))
    output = [ [ [ [ 0 for i in range(n_cols - width + 1) ] for j in range(n_rows - height + 1) ] for k in range(n_filters) ] for l in range(n_imgs) ]
    for ii in range(n_imgs):
        for rr in range(n_rows - height + 1):
            for cc in range(n_cols - width + 1):
                for hh in range(height):
                    for ww in range(width):
                        for jj in range(n_channels):
                            for ff in range(n_filters):
                                imgval = imgs[ii][rr + hh][cc + ww][jj]
                                filterval = filters[ff][hh][ww][jj]
                                output[ii][ff][rr][cc] += imgval * filterval
    return output



#!/usr/bin/env python
# taken from http://www.ripton.net/blog/?p=51
#pythran export euler14(int)
#runas euler14(1000)
#bench euler14(650000)

"""Project Euler, problem 14

The following iterative sequence is defined for the set of positive integers:

n -> n / 2 (n is even)
n -> 3n + 1 (n is odd)

Which starting number, under one million, produces the longest chain?
"""

def next_num(num):
    if num & 1:
        return 3 * num + 1
    else:
        return num // 2



def series_length(num, lengths):
    if num in lengths:
        return lengths[num]
    else:
        num2 = next_num(num)
        result = 1 + series_length(num2, lengths)
        lengths[num] = result
        return result

def euler14(MAX_NUM):
    num_with_max_length = 1
    max_length = 0
    lengths = {1: 0}
    for ii in range(1, MAX_NUM):
        length = series_length(ii, lengths)
        if length > max_length:
            max_length = length
            num_with_max_length = ii
    return num_with_max_length, max_length



#runas solve()
#pythran export solve()
def solve():
    '''
    Pentagonal numbers are generated by the formula, P_n=n(3n-1)/2. The first ten pentagonal numbers are:
    
    1, 5, 12, 22, 35, 51, 70, 92, 117, 145, ...
    
    It can be seen that P_4 + P_7 = 22 + 70 = 92 = P_8. However, their difference, 70 - 22 = 48, is not pentagonal.
    
    Find the pair of pentagonal numbers, P_j and P_k, for which their sum and difference is pentagonal and D = |P_k - P_j| is minimised; what is the value of D?
    '''

    MAX = 2000
    pent = [ n * (3*n - 1) / 2 for n in range(1, 2*MAX) ]
    pdic = dict.fromkeys(pent)

    def main2():
        for j in range(0, MAX):
            for k in range(j+1, 2*MAX-1):
                p_j = pent[j]
                p_k = pent[k]
                p_sum = p_j + p_k
                p_diff = p_k - p_j
                if p_sum in pdic and p_diff in pdic:
                    return p_diff

    return main2()




import numpy as np

# pythran export possible_hull(uint8[:,:])


def possible_hull(img):
    """Return positions of pixels that possibly belong to the convex hull.

    Parameters
    ----------
    img : ndarray of bool
        Binary input image.

    Returns
    -------
    coords : ndarray (cols, 2)
       The ``(row, column)`` coordinates of all pixels that possibly belong to
       the convex hull.

    """
    rows, cols = img.shape

    # Output: rows storage slots for left boundary pixels
    #         cols storage slots for top boundary pixels
    #         rows storage slots for right boundary pixels
    #         cols storage slots for bottom boundary pixels
    coords = np.ones((2 * (rows + cols), 2), dtype=np.intp)
    coords *= -1

    rows_cols = rows + cols
    rows_2_cols = 2 * rows + cols

    for r in range(rows):

        rows_cols_r = rows_cols + r

        for c in range(cols):

            if img[r, c] != 0:

                rows_c = rows + c
                rows_2_cols_c = rows_2_cols + c

                # Left check
                if coords[r, 1] == -1:
                    coords[r, 0] = r
                    coords[r, 1] = c

                # Right check
                elif coords[rows_cols_r, 1] < c:
                    coords[rows_cols_r] = r, c

                # Top check
                if coords[rows_c, 1] == -1:
                    coords[rows_c] = r, c

                # Bottom check
                elif coords[rows_2_cols_c, 0] < r:
                    coords[rows_2_cols_c] = r, c

    return coords[coords[:, 0] != -1]



#runas a=[ [ float(i) for i in range(60)] for j in range(60)] ; matrix_multiply(a,a)
#runas a=[ [ float(i) for i in range(60)] for j in range(40)] ; b=[ [ float(i) for i in range(40)] for j in range(60)]; matrix_multiply(a,b)
#pythran export matrix_multiply(float list list, float list list)
def zero(n,m): return [[0 for row in range(n)] for col in range(m)]
def matrix_multiply(m0, m1):
    new_matrix = zero(len(m0),len(m1[0]))
    for i in range(len(m0)):
        for j in range(len(m1[0])):
            r=0
            "omp parallel for reduction(+:r)"
            for k in range(len(m1)):
                r += m0[i][k]*m1[k][j]
            new_matrix[i][j]=r
    return new_matrix



def omp_task_shared():
    i = 0
    k = 0
    result = 0
    NUM_TASKS = 25

    if 'omp parallel':
        if 'omp single':
            for k in range(0, NUM_TASKS):
                if 'omp task shared(i)':
                    'omp atomic'
                    i += 1
    result = i
    return result == NUM_TASKS



#from https://gist.github.com/andersx/6061586
#runas run()
#bench run()
#pythran export run()

# A simple energy minimization program that uses steepest descent 
# and a force field to minimize the energy of water in internal coordinates.
# Written by Jan H. Jensen, 2013


def Eandg(rOH,thetaHOH):

    """"
    Arguments: (internal coordinates of the water molecule)

    rOH            O-H bond distance
    thetaHOH       H-O-H bond angle


    Returns:

    E              Molecular force field energy
    grOH           O-H bond stretch gradient
    grthetaHOH     H-O-H bond angle bend gradient 

    
    Force field parameters:

    kOH            Harmonic force constant, O-H bond strech
    rOHe           Equilibrium distance, O-H
    kHOH           Harmonic angle bend force constant, H-O-H angle bend
    thetaHOHe      Equilibrium angle, H-O-H

    """

    kOH = 50.0
    rOHe = 0.95
    kHOH = 50.0
    thetaHOHe = 104.5


    E = 2 * kOH * (rOH - rOHe)**2 + kHOH * (thetaHOH - thetaHOHe)**2
    grOH = 2 * kOH * (rOH - rOHe)
    grthetaHOH = 2 * kHOH * (thetaHOH - thetaHOHe)

    return (E, grOH, grthetaHOH)

def run():
    c = 0.005
    n_steps = 1000000

    #starting geometry
    rOH = 10.0
    thetaHOH = 180.0



    for i in range(n_steps):
        (E,grOH,gthetaHOH) = Eandg(rOH,thetaHOH)
        if (abs(grOH) >0.001/c or abs(gthetaHOH) > 0.01/c ):
            rOH = rOH - c*grOH
            thetaHOH = thetaHOH - c*gthetaHOH

    converged = (abs(grOH) >0.001/c or abs(gthetaHOH) > 0.01/c )

    return converged, E,rOH,thetaHOH



import numpy as np


def linear(r):
    return -r


def thin_plate_spline(r):
    if r == 0:
        return 0.0
    else:
        return r**2*np.log(r)


def cubic(r):
    return r**3


def quintic(r):
    return -r**5


def multiquadric(r):
    return -np.sqrt(r**2 + 1)


def inverse_multiquadric(r):
    return 1/np.sqrt(r**2 + 1)


def inverse_quadratic(r):
    return 1/(r**2 + 1)


def gaussian(r):
    return np.exp(-r**2)


NAME_TO_FUNC = {
   "linear": linear,
   "thin_plate_spline": thin_plate_spline,
   "cubic": cubic,
   "quintic": quintic,
   "multiquadric": multiquadric,
   "inverse_multiquadric": inverse_multiquadric,
   "inverse_quadratic": inverse_quadratic,
   "gaussian": gaussian
   }


def kernel_vector(x, y, kernel_func, out):
    """Evaluate RBFs, with centers at `y`, at the point `x`."""
    for i in range(y.shape[0]):
        out[i] = kernel_func(np.linalg.norm(x - y[i]))


def polynomial_vector(x, powers, out):
    """Evaluate monomials, with exponents from `powers`, at the point `x`."""
    for i in range(powers.shape[0]):
        out[i] = np.prod(x**powers[i])


def kernel_matrix(x, kernel_func, out):
    """Evaluate RBFs, with centers at `x`, at `x`."""
    for i in range(x.shape[0]):
        for j in range(i+1):
            out[i, j] = kernel_func(np.linalg.norm(x[i] - x[j]))
            out[j, i] = out[i, j]


def polynomial_matrix(x, powers, out):
    """Evaluate monomials, with exponents from `powers`, at `x`."""
    for i in range(x.shape[0]):
        for j in range(powers.shape[0]):
            out[i, j] = np.prod(x[i]**powers[j])


# pythran export _kernel_matrix(float[:, :], str)
def _kernel_matrix(x, kernel):
    """Return RBFs, with centers at `x`, evaluated at `x`."""
    out = np.empty((x.shape[0], x.shape[0]), dtype=float)
    kernel_func = NAME_TO_FUNC[kernel]
    kernel_matrix(x, kernel_func, out)
    return out


# pythran export _polynomial_matrix(float[:, :], int[:, :])
def _polynomial_matrix(x, powers):
    """Return monomials, with exponents from `powers`, evaluated at `x`."""
    out = np.empty((x.shape[0], powers.shape[0]), dtype=float)
    polynomial_matrix(x, powers, out)
    return out


# pythran export _build_system(float[:, :],
#                              float[:, :],
#                              float[:],
#                              str,
#                              float,
#                              int[:, :])
def _build_system(y, d, smoothing, kernel, epsilon, powers):
    """Build the system used to solve for the RBF interpolant coefficients.
    Parameters
    ----------
    y : (P, N) float ndarray
        Data point coordinates.
    d : (P, S) float ndarray
        Data values at `y`.
    smoothing : (P,) float ndarray
        Smoothing parameter for each data point.
    kernel : str
        Name of the RBF.
    epsilon : float
        Shape parameter.
    powers : (R, N) int ndarray
        The exponents for each monomial in the polynomial.
    Returns
    -------
    lhs : (P + R, P + R) float ndarray
        Left-hand side matrix.
    rhs : (P + R, S) float ndarray
        Right-hand side matrix.
    shift : (N,) float ndarray
        Domain shift used to create the polynomial matrix.
    scale : (N,) float ndarray
        Domain scaling used to create the polynomial matrix.
    """
    p = d.shape[0]
    s = d.shape[1]
    r = powers.shape[0]
    kernel_func = NAME_TO_FUNC[kernel]

    # Shift and scale the polynomial domain to be between -1 and 1
    mins = np.min(y, axis=0)
    maxs = np.max(y, axis=0)
    shift = (maxs + mins)/2
    scale = (maxs - mins)/2
    # The scale may be zero if there is a single point or all the points have
    # the same value for some dimension. Avoid division by zero by replacing
    # zeros with ones.
    scale[scale == 0.0] = 1.0

    yeps = y*epsilon
    yhat = (y - shift)/scale

    # Transpose to make the array fortran contiguous. This is required for
    # dgesv to not make a copy of lhs.
    lhs = np.empty((p + r, p + r), dtype=float).T
    kernel_matrix(yeps, kernel_func, lhs[:p, :p])
    polynomial_matrix(yhat, powers, lhs[:p, p:])
    lhs[p:, :p] = lhs[:p, p:].T
    lhs[p:, p:] = 0.0
    for i in range(p):
        lhs[i, i] += smoothing[i]

    # Transpose to make the array fortran contiguous.
    rhs = np.empty((s, p + r), dtype=float).T
    rhs[:p] = d
    rhs[p:] = 0.0

    return lhs, rhs, shift, scale


# pythran export _evaluate(float[:, :],
#                          float[:, :],
#                          str,
#                          float,
#                          int[:, :],
#                          float[:],
#                          float[:],
#                          float[:, :])
def _evaluate(x, y, kernel, epsilon, powers, shift, scale, coeffs):
    """Evaluate the RBF interpolant at `x`.
    Parameters
    ----------
    x : (Q, N) float ndarray
        Interpolation point coordinates.
    y : (P, N) float ndarray
        Data point coordinates.
    kernel : str
        Name of the RBF.
    epsilon : float
        Shape parameter.
    powers : (R, N) int ndarray
        The exponents for each monomial in the polynomial.
    shift : (N,) float ndarray
        Shifts the polynomial domain for numerical stability.
    scale : (N,) float ndarray
        Scales the polynomial domain for numerical stability.
    coeffs : (P + R, S) float ndarray
        Coefficients for each RBF and monomial.
    Returns
    -------
    (Q, S) float ndarray
    """
    q = x.shape[0]
    p = y.shape[0]
    r = powers.shape[0]
    s = coeffs.shape[1]
    kernel_func = NAME_TO_FUNC[kernel]

    yeps = y*epsilon
    xeps = x*epsilon
    xhat = (x - shift)/scale

    out = np.zeros((q, s), dtype=float)
    vec = np.empty((p + r,), dtype=float)
    for i in range(q):
        kernel_vector(xeps[i], yeps, kernel_func, vec[:p])
        polynomial_vector(xhat[i], powers, vec[p:])
        # Compute the dot product between coeffs and vec. Do not use np.dot
        # because that introduces build complications with BLAS (see
        # https://github.com/serge-sans-paille/pythran/issues/1346)
        for j in range(s):
            for k in range(p + r):
                out[i, j] += coeffs[k, j]*vec[k]

    return out



import unittest
from pythran.tests import TestFromDir
import os
import glob
import pythran

class TestUserImport(TestFromDir):
    path = os.path.join(os.path.dirname(__file__),"user_defined_import")
    files = glob.glob(os.path.join(path,"*_main.py"))
    files.append(os.path.join(path, "tiny_project", "level", "dummy.py"))


TestUserImport.populate(TestUserImport)

if __name__ == '__main__':
    unittest.main()



#pythran export insertion_sort(float list)
#runas insertion_sort([1.3,5.6,-34.4,34.4,32,1.2,0,0.0,3.4,1.3])
#bench import random; in_ = random.sample(range(10000000), 6000) + [4.5]; insertion_sort(in_)
def insertion_sort(list2):
    for i in range(1, len(list2)):
        save = list2[i]
        j = i
        while j > 0 and list2[j - 1] > save:
            list2[j] = list2[j - 1]
            j -= 1
        list2[j] = save



#runas solve(2)
#pythran export solve(int)
def solve(digit):
    '''
    The fraction 49/98 is a curious fraction, as an inexperienced mathematician in attempting to simplify it may incorrectly believe that 49/98 = 4/8, which is correct, is obtained by cancelling the 9s.

    We shall consider fractions like, 30/50 = 3/5, to be trivial examples.

    There are exactly four non-trivial examples of this type of fraction, less than one in value, and containing two digits in the numerator and denominator.

    If the product of these four fractions is given in its lowest common terms, find the value of the denominator.
    '''

    def fractions():
        for numerator in map(str, range(10 ** (digit - 1), 10 ** digit)):
            for denominator in map(str, range(int(numerator)+1, 10 ** digit)):
                if numerator == denominator: continue
                if numerator[1] == denominator[1] and numerator[1] == '0': continue
                if numerator[0] == denominator[0] and int(numerator) * int(denominator[1]) == int(denominator) * int(numerator[1]): yield(int(numerator), int(denominator))
                if numerator[0] == denominator[1] and int(numerator) * int(denominator[0]) == int(denominator) * int(numerator[1]): yield(int(numerator), int(denominator))
                if numerator[1] == denominator[1] and int(numerator) * int(denominator[0]) == int(denominator) * int(numerator[0]): yield(int(numerator), int(denominator))
                if numerator[1] == denominator[0] and int(numerator) * int(denominator[1]) == int(denominator) * int(numerator[0]): yield(int(numerator), int(denominator))

    def gcd(a,b): return b and gcd(b, a % b) or a

    numerator = 1
    denominator = 1
    for frac in fractions():
        numerator = numerator * frac[0]
        denominator = denominator * frac[1]

    g = gcd(numerator, denominator)
    return denominator / g




#from http://rosettacode.org/wiki/Permutations#Python
#pythran export test(int)
#runas test(3)

def test(n):
    import itertools
    return [values for values in itertools.permutations(range(1, n + 1))]



def omp_task_untied():
    import omp
    from time import sleep
    NUM_TASKS = 25
    start_id = [0 for _ in range(NUM_TASKS)]
    current_id = [0 for _ in range(NUM_TASKS)]
    count = 0

    if 'omp parallel':
        use_parallel = omp.in_parallel()
        if 'omp single':
            for i in range(NUM_TASKS):
                myi = i
                if 'omp task firstprivate(myi) untied':
                    sleep(0.01)
                    start_id[myi] = omp.get_thread_num()

                    'omp taskwait'

                    if start_id[myi] % 2 != 0:
                        sleep(0.01)
                        current_id[myi] = omp.get_thread_num()

    for i in range(NUM_TASKS):
        if current_id[i] == start_id[i]:
            count += 1

    return count < NUM_TASKS or not use_parallel



from pythran.tests import TestEnv
from pythran.typing import NDArray, Tuple, List

import numpy
from distutils.version import LooseVersion

import unittest

try:
    numpy.float128
    has_float128 = True
except AttributeError:
    has_float128 = False


huge = numpy.iinfo(numpy.intp).max // numpy.intp().itemsize

def raisesMemoryError():
    try:
        numpy.ones(huge)
        return False
    except MemoryError:
        return True
    except ValueError:
        return False


@TestEnv.module
class TestNdarray(TestEnv):

    @unittest.skipIf(not raisesMemoryError(), "memory error not triggered on that arch")
    def test_ndarray_memory_error(self):
        code = 'def ndarray_memory_error(n): import numpy as np; return np.ones(n)'
        self.run_test(code, huge,
                      ndarray_memory_error=[numpy.intp],
                      check_exception=True)

    def test_ndarray_intc(self):
        self.run_test('def ndarray_intc(a): import numpy as np; return np.intc(a), np.array([a, a], dtype=np.intc)',
                      numpy.intc(5),
                      ndarray_intc=[numpy.intc])

    def test_ndarray_uintc(self):
        self.run_test('def ndarray_uintc(a): import numpy as np; return np.uintc(a), np.array([a, a], dtype=np.uintc)',
                      numpy.uintc(5),
                      ndarray_uintc=[numpy.uintc])

    def test_ndarray_intp(self):
        self.run_test('def ndarray_intp(a): import numpy as np; return np.intp(a), np.array([a, a], dtype=np.intp)',
                      numpy.intp(-5),
                      ndarray_intp=[numpy.intp])

    def test_ndarray_uintp(self):
        self.run_test('def ndarray_uintp(a): import numpy as np; return np.uintp(a), np.array([a, a], dtype=np.uintp)',
                      numpy.uintp(5),
                      ndarray_uintp=[numpy.uintp])

    def test_ndarray_real_attr_read(self):
        self.run_test('def ndarray_real_attr_read(a): return a.real + 1',
                      numpy.arange(100, dtype=numpy.complex128).reshape((10, 10)),
                      ndarray_real_attr_read=[NDArray[complex, :, :]])

    def test_ndarray_imag_attr_read(self):
        self.run_test('def ndarray_imag_attr_read(a): return a.imag + 1',
                      1j * numpy.arange(10, dtype=numpy.complex128),
                      ndarray_imag_attr_read=[NDArray[complex, :]])

    def test_ndarray_real_attr_read_complex64(self):
        self.run_test('def ndarray_real_attr_read_complex64(a): return a.real + 1',
                      numpy.arange(100, dtype=numpy.complex64).reshape((10, 10)),
                      ndarray_real_attr_read_complex64=[NDArray[numpy.complex64, :, :]])

    def test_ndarray_imag_attr_read_complex64(self):
        self.run_test('def ndarray_imag_attr_read_complex64(a): return a.imag + 1',
                      1j * numpy.arange(10, dtype=numpy.complex64),
                      ndarray_imag_attr_read_complex64=[NDArray[numpy.complex64, :]])

    def test_ndarray_real_attr_write(self):
        self.run_test('def ndarray_real_attr_write(a): a.real = 1 ; return a',
                      numpy.arange(100, dtype=numpy.complex128).reshape((10, 10)),
                      ndarray_real_attr_write=[NDArray[complex, :, :]])

    def test_ndarray_imag_attr_write(self):
        self.run_test('def ndarray_imag_attr_write(a): a.imag = 1 ; return a',
                      1j * numpy.arange(10, dtype=numpy.complex128),
                      ndarray_imag_attr_write=[NDArray[complex, :]])

    def test_ndarray_real_fun_read(self):
        self.run_test('def ndarray_real_fun_read(a): import numpy as np ; return np.real(a)[1:]',
                      numpy.arange(100, dtype=numpy.complex128).reshape((10, 10)),
                      ndarray_real_fun_read=[NDArray[complex, :, :]])

    def test_ndarray_imag_fun_read(self):
        self.run_test('def ndarray_imag_fun_read(a): import numpy as np ; return - np.imag(a)',
                      1j * numpy.arange(10, dtype=numpy.complex128),
                      ndarray_imag_fun_read=[NDArray[complex, :]])

    def test_ndarray_real_vexpr_read(self):
        self.run_test('def ndarray_real_vexpr_read(a): import numpy as np ; return a[np.argsort(a)].real',
                      numpy.arange(10, dtype=numpy.complex128),
                      ndarray_real_vexpr_read=[NDArray[complex, :]])

    def test_ndarray_imag_vexpr_read(self):
        self.run_test('def ndarray_imag_vexpr_read(a): import numpy as np ; return a[np.argsort(a)].imag',
                      1j * numpy.arange(10, dtype=numpy.complex128),
                      ndarray_imag_vexpr_read=[NDArray[complex, :]])

    def test_numpy_augassign0(self):
        self.run_test('def numpy_augassign0(a): a+=1; return a',
                      numpy.arange(100).reshape((10, 10)),
                      numpy_augassign0=[NDArray[int, :, :]])

    def test_numpy_augassign1(self):
        self.run_test('def numpy_augassign1(a): a*=2; return a',
                      numpy.arange(100).reshape((10, 10)),
                      numpy_augassign1=[NDArray[int, :, :]])

    def test_numpy_augassign2(self):
        self.run_test('def numpy_augassign2(a): a-=2; return a',
                      numpy.arange(100).reshape((10, 10)),
                      numpy_augassign2=[NDArray[int, :, :]])

    def test_numpy_augassign3(self):
        self.run_test('def numpy_augassign3(a): a//=2; return a',
                      numpy.arange(100).reshape((10, 10)),
                      numpy_augassign3=[NDArray[int, :, :]])

    def test_numpy_augassign4(self):
        self.run_test('def numpy_augassign4(a): a|=2; return a',
                      numpy.arange(100).reshape((10, 10)),
                      numpy_augassign4=[NDArray[int, :, :]])

    def test_numpy_augassign5(self):
        self.run_test('def numpy_augassign5(a): a&=2; return a',
                      numpy.arange(100).reshape((10, 10)),
                      numpy_augassign5=[NDArray[int, :, :]])

    def test_numpy_augassign6(self):
        self.run_test('def helper(x): x //= 2; x+=3\ndef numpy_augassign6(a): a&=2; helper(a); return a',
                      numpy.arange(100).reshape((10, 10)),
                      numpy_augassign6=[NDArray[int, :, :]])

    def test_numpy_faugassign0(self):
        self.run_test('def numpy_faugassign0(a): a[a>5]+=1; return a',
                      numpy.arange(100),
                      numpy_faugassign0=[NDArray[int, :]])

    def test_numpy_faugassign1(self):
        self.run_test('def numpy_faugassign1(a): a[a>3]*=2; return a',
                      numpy.arange(100),
                      numpy_faugassign1=[NDArray[int, :]])

    def test_numpy_faugassign2(self):
        self.run_test('def numpy_faugassign2(a): a[a>30]-=2; return a',
                      numpy.arange(100),
                      numpy_faugassign2=[NDArray[int, :]])

    def test_numpy_faugassign3(self):
        self.run_test('def numpy_faugassign3(a): a[a<40]//=2; return a',
                      numpy.arange(100),
                      numpy_faugassign3=[NDArray[int, :]])

    def test_numpy_faugassign4(self):
        self.run_test('def numpy_faugassign4(a): a[a<4]|=2; return a',
                      numpy.arange(100),
                      numpy_faugassign4=[NDArray[int, :]])

    def test_numpy_faugassign5(self):
        self.run_test('def numpy_faugassign5(a): a[a>8]&=2; return a',
                      numpy.arange(100),
                      numpy_faugassign5=[NDArray[int, :]])

    def test_broadcast0(self):
        self.run_test('def numpy_broadcast0(a): a[0] = 1 ; return a',
                      numpy.arange(100).reshape((10, 10)),
                      numpy_broadcast0=[NDArray[int, :, :]])

    def test_broadcast1(self):
        self.run_test('def numpy_broadcast1(a): a[1:-1] = 1 ; return a',
                      numpy.arange(100).reshape((10, 10)),
                      numpy_broadcast1=[NDArray[int, :, :]])

    def test_broadcast2(self):
        self.run_test('def numpy_broadcast2(a): a[1:-1,1:-1] = 1 ; return a',
                      numpy.arange(100).reshape((10, 10)),
                      numpy_broadcast2=[NDArray[int, :, :]])

    def test_broadcast3(self):
        self.run_test('def numpy_broadcast3(a): a[1:-1,1] = 1 ; return a',
                      numpy.arange(100).reshape((10, 10)),
                      numpy_broadcast3=[NDArray[int, :, :]])

    def test_broadcast4(self):
        self.run_test('def numpy_broadcast4(a): a[:,1,1] = 1 ; return a',
                      numpy.arange(100).reshape((5,5,4)),
                      numpy_broadcast4=[NDArray[int, :, :, :]])

    def test_broadcast5(self):
        self.run_test('def numpy_broadcast5(a): import numpy as np ; return a + np.array([1,2,3,4])',
                      numpy.arange(20).reshape((5,4)),
                      numpy_broadcast5=[NDArray[int, :, :]])

    def test_extended_slicing0(self):
        self.run_test("def numpy_extended_slicing0(a): return a[2,1:-1]",
                      numpy.arange(100).reshape((10, 10)),
                      numpy_extended_slicing0=[NDArray[int, :, :]])

    def test_extended_slicing1(self):
        self.run_test("def numpy_extended_slicing1(a): return a[1:-1,2]",
                      numpy.arange(100).reshape((10, 10)),
                      numpy_extended_slicing1=[NDArray[int, :, :]])

    def test_extended_slicing2(self):
        self.run_test("def numpy_extended_slicing2(a): return a[2,1:-1]",
                      numpy.arange(30).reshape((3,5,2)),
                      numpy_extended_slicing2=[NDArray[int, :, :, :]])

    def test_extended_slicing3(self):
        self.run_test("def numpy_extended_slicing3(a): return a[1:-1,2]",
                      numpy.arange(30).reshape((3,5,2)),
                      numpy_extended_slicing3=[NDArray[int, :,:,:]])

    def test_extended_slicing4(self):
        self.run_test("def numpy_extended_slicing4(a): return a[1:-1,2:-2]",
                      numpy.arange(100).reshape((10, 10)),
                      numpy_extended_slicing4=[NDArray[int, :, :]])

    def test_extended_slicing5(self):
        self.run_test("def numpy_extended_slicing5(a): return a[1:-1]",
                      numpy.arange(100).reshape((10, 10)),
                      numpy_extended_slicing5=[NDArray[int, :, :]])

    def test_extended_slicing6(self):
        self.run_test("def numpy_extended_slicing6(a): return a[1:-1,2:-2, 3:-3]",
                      numpy.arange(5*6*7).reshape((5,6,7)),
                      numpy_extended_slicing6=[NDArray[int,:,:,:]])

    def test_extended_slicing7(self):
        self.run_test("def numpy_extended_slicing7(a): return a[1:-1, 2, 1]",
                      numpy.arange(120).reshape((3,5,4,2)),
                      numpy_extended_slicing7=[NDArray[int,:,:,:,:]])

    def test_extended_slicing8(self):
        self.run_test("def numpy_extended_slicing8(a): return a[1:-1,2:-2, 1:2]",
                      numpy.arange(60).reshape((3,5,4)),
                      numpy_extended_slicing8=[NDArray[int,:,:,:]])

    def test_extended_slicing9(self):
        self.run_test("def numpy_extended_slicing9(a): return a[1:-1, 2, 1, 1:2]",
                      numpy.arange(120).reshape((3,5,2,4)),
                      numpy_extended_slicing9=[NDArray[int,:,:,:,:]])

    def test_extended_slicing10(self):
        self.run_test("def numpy_extended_slicing10(a): return a[1, 2, 1:-1]",
                      numpy.arange(120).reshape((3,5,4,2)),
                      numpy_extended_slicing10=[NDArray[int,:,:,:,:]])

    def test_extended_slicing11(self):
        self.run_test("def numpy_extended_slicing11(a): return a[1, 2, 1:-1, 1]",
                      numpy.arange(120).reshape((3,5,4,2)),
                      numpy_extended_slicing11=[NDArray[int,:,:,:,:]])

    def test_numpy_mask0(self):
        self.run_test("def numpy_mask0(n): import numpy ; return n[n>0][ n[n>0] < 1]",
                      numpy.cos(numpy.arange(10)),
                      numpy_mask0=[NDArray[float, :]])

    def test_numpy_bool(self):
        self.run_test("def numpy_bool(n): import numpy ; return numpy.ones(n, bool)",
                      5,
                      numpy_bool=[int])

    def test_numpy_int(self):
        self.run_test("def numpy_int(n): import numpy ; return numpy.ones(n, int)",
                      5,
                      numpy_int=[int])

    def test_numpy_float(self):
        self.run_test("def numpy_float(n): import numpy ; return numpy.ones(n, float)",
                      5,
                      numpy_float=[int])

    def test_numpy_int16(self):
        self.run_test("def numpy_int16(n): import numpy ; return numpy.ones(n, numpy.int16)",
                      5,
                      numpy_int16=[int])

    def test_numpy_uint16(self):
        self.run_test("def numpy_uint16(n): import numpy ; return numpy.ones(n, numpy.uint16)",
                      5,
                      numpy_uint16=[int])

    def test_numpy_uint64(self):
        self.run_test("def numpy_uint64(n): import numpy ; return numpy.ones(n, numpy.uint64)",
                      5,
                      numpy_uint64=[int])

    def test_numpy_np_float(self):
        """ Check dtype == numpy.float for numpy array. """
        self.run_test("""
            def numpy_np_float(n):
                import numpy
                return numpy.ones(n, float)""",
                      5,
                      numpy_np_float=[int])

    def test_numpy_complex(self):
        self.run_test("def numpy_complex(n): import numpy ; return numpy.ones(n, complex)",
                      5,
                      numpy_complex=[int])

    def test_numpy_complex64(self):
        self.run_test("def numpy_complex64(n): import numpy ; return numpy.ones(n, numpy.complex64)",
                      5,
                      numpy_complex64=[int])

    def test_numpy_double(self):
        self.run_test("def numpy_double(n): import numpy ; return numpy.ones(n, numpy.double)",
                      5,
                      numpy_double=[int])

    def test_numpy_complex_export(self):
        self.run_test("def numpy_complex_export(a): import numpy ; return numpy.sum(a)",
                      numpy.array([1+1j]),
                      numpy_complex_export=[NDArray[complex, :]])

    def test_assign_gsliced_array(self):
        self.run_test("""def assign_gsliced_array():
   import numpy as np;
   a = np.array([[[1,2],[3,4]],[[5,6],[7,8]]])
   b = np.array([[[9,10],[11,12]],[[13,14],[15,16]]])
   a[:,:] = b[:,:]
   return a,b;""", assign_gsliced_array=[])

    def test_assign_sliced_array(self):
        self.run_test("""def assign_sliced_array():
   import numpy as np;
   a = np.array([1,2,3]);
   b = np.array([1,2,3]);
   c=a[1:]
   c=b[1:]
   b[2] = -1;
   return c;""", assign_sliced_array=[])

    def test_index_array_0(self):
        self.run_test('''
            def index_array_0(n):
                import numpy
                a = numpy.arange(n)
                return a[numpy.array([1, 0])]''',
                      10,
                      index_array_0=[int])

    def test_index_array_1(self):
        self.run_test('''
            def index_array_1(n):
                import numpy
                a = numpy.arange(n * 3).reshape(3, n)
                return a[numpy.array([0, 1, 0, 2])]''',
                      10,
                      index_array_1=[int])

    def test_filter_array_0(self):
        self.run_test('def filter_array_0(n): import numpy ; a = numpy.zeros(n) ; return a[a>1]',
                      10,
                      filter_array_0=[int])

    def test_filter_array_1(self):
        self.run_test('def filter_array_1(n): import numpy ; a = numpy.arange(n) ; return a[a>4]',
                      10,
                      filter_array_1=[int])

    def test_filter_array_2(self):
        self.run_test('def filter_array_2(n): import numpy ; a = numpy.arange(n) ; return (a+a)[a>4]',
                      10,
                      filter_array_2=[int])

    def test_filter_array_3(self):
        self.run_test('def filter_array_3(n): import numpy ; a = numpy.arange(n) ; return (-a)[a>4]',
                      10,
                      filter_array_3=[int])

    @unittest.skip("filtering a slice")
    def test_filter_array_4(self):
        self.run_test('def filter_array_4(n): import numpy ; a = numpy.arange(n) ; return a[1:-1][a[1:-1]>4]',
                      10,
                      filter_array_4=[int])

    @unittest.skip("filtering a slice")
    def test_filter_array_5(self):
        self.run_test('def filter_array_5(n): import numpy ; a = numpy.arange(n) ; return (a[1:-1])[a[1:-1]>4]',
                      10,
                      filter_array_5=[int])

    def test_assign_ndarray(self):
        code = """
def assign_ndarray(t):
    import numpy as np;
    a = np.array([1,2,3]);
    b = np.array([1,2,3]);
    if t:
      c = a;
    else:
      c=b;
    if t:
      c=b;
    b[0] = -1;
    return c;"""
        self.run_test(code,
                      1,
                      assign_ndarray=[int])

    def test_bitwise_nan_bool(self):
        self.run_test("def np_bitwise_nan_bool(a): import numpy as np ; return ~(a<5)", numpy.arange(10), np_bitwise_nan_bool=[NDArray[int, :]])

    def test_gslice0(self):
        self.run_test("def np_gslice0(a): import numpy as np; return a[1:9,5:7]", numpy.array(range(10*9)).reshape((10,9)), np_gslice0=[NDArray[int, :, :]])

    def test_gslice1(self):
        self.run_test("def np_gslice1(a): import numpy as np ; return a[1:9,0:1, 3:6]", numpy.array(range(10*9*8)).reshape((10,9,8)), np_gslice1=[NDArray[int, :, :, :]])

    def test_gslice2(self):
        self.run_test("def np_gslice2(a): import numpy as np ; return a[:,0:1, 3:6]", numpy.array(range(10*9*8)).reshape((10,9,8)), np_gslice2=[NDArray[int, :, :, :]])

    def test_gslice3(self):
        self.run_test("def np_gslice3(a): import numpy as np ; return a[:-1,0:-1, -3:7]", numpy.array(range(10*9*8)).reshape((10,9,8)), np_gslice3=[NDArray[int, :, :, :]])

    def test_gslice4(self):
        self.run_test("def np_gslice4(a): import numpy as np ; return a[1,0:-1, -3:7]", numpy.array(range(10*9*8)).reshape((10,9,8)), np_gslice4=[NDArray[int, :, :, :]])

    def test_gslice5(self):
        self.run_test("def np_gslice5(a): import numpy as np ; return a[1,0:-1, 7]", numpy.array(range(10*9*8)).reshape((10,9,8)), np_gslice5=[NDArray[int, :, :, :]])

    def test_gslice6(self):
        self.run_test("def np_gslice6(a): import numpy as np ; return a[:-1, :][1:,:]", numpy.array(range(10*9*8)).reshape((10,9,8)), np_gslice6=[NDArray[int, :,:,:]])

    def test_iexpr0(self):
        self.run_test("def np_iexpr0(a,i): return a[i][0,0]", numpy.array(range(10*9*8)).reshape(10,9,8), 0, np_iexpr0=[NDArray[int, :,:,:], int])

    def test_iexpr1(self):
        self.run_test("def np_iexpr1(a,i): return a[i,0][0]", numpy.array(range(10*9*8)).reshape(10,9,8), 0, np_iexpr1=[NDArray[int, :,:,:], int])

    def test_iexpr2(self):
        self.run_test("def np_iexpr2(a,m): a[m==False] = 1; return a", numpy.arange(10).reshape(5,2), numpy.arange(10).reshape(5,2), np_iexpr2=[NDArray[int, :,:], NDArray[int, :,:]])

    def test_iexpr3(self):
        code = '''
import numpy as np
def np_iexpr3 (x, nbits):
    out = np.zeros ((len(x), 1<<nbits), dtype=complex)
    for e in range (len (x)):
        out[e,x[e]] = 1
    return out'''
        self.run_test(code, numpy.arange(16, dtype=numpy.int16), 4, np_iexpr3=[NDArray[numpy.int16, :], int])

    def test_item0(self):
        self.run_test("def np_item0(a): return a.item(3)", numpy.array([[3, 1, 7],[2, 8, 3],[8, 5, 3]]), np_item0=[NDArray[int, :,: ]])

    def test_item1(self):
        self.run_test("def np_item1(a): return a.item(7)", numpy.array([[3, 1, 7],[2, 8, 3],[8, 5, 3]]), np_item1=[NDArray[int, :,: ]])

    def test_item2(self):
        self.run_test("def np_item2(a): return a.item((0,1))",numpy.array([[3, 1, 7],[2, 8, 3],[8, 5, 3]]),  np_item2=[NDArray[int, :,: ]])

    def test_item3(self):
        self.run_test("def np_item3(a): return a.item((2,2))", numpy.array([[3, 1, 7],[2, 8, 3],[8, 5, 3]]), np_item3=[NDArray[int, :,: ]])

    def test_item4(self):
        self.run_test("def np_item4(a): return a.item(-2)", numpy.array([[3, 1, 7],[2, 8, 3],[8, 5, 3]]), np_item4=[NDArray[int, :,: ]])

    def test_acces1D_(self):
        self.run_test("def np_acces1D_(a): return a[1]", numpy.array([1,2,3]), np_acces1D_=[NDArray[int, :]])

    def test_accesSimple_(self):
        self.run_test("def np_accesSimple_(a): return a[1]", numpy.array([[1,2],[3,4]]), np_accesSimple_=[NDArray[int, :,: ]])

    def test_accesMultiple_(self):
        self.run_test("def np_accesMultiple_(a): return a[1,0]", numpy.array([[1,2],[3,4]]), np_accesMultiple_=[NDArray[int, :,: ]])

    def test_accesMultipleND_(self):
        self.run_test("def np_accesMultipleND_(a): return a[1,0]", numpy.array([[[1,2],[3,4]],[[5,6],[7,8]]]), np_accesMultipleND_=[NDArray[int, :,:,:]])

    def test_accesMultipleNDSplit_(self):
        self.run_test("def np_accesMultipleNDSplit_(a): return a[1][0]", numpy.array([[[1,2],[3,4]],[[5,6],[7,8]]]), np_accesMultipleNDSplit_=[NDArray[int, :,:,:]])

    def test_shape_(self):
        self.run_test("def np_shape_(a): return a.shape", numpy.array([[[1,2],[3,4]],[[5,6],[7,8]]]), np_shape_=[NDArray[int, :,:,:]])

    def test_change_arrayND_(self):
        self.run_test("def np_change_arrayND_(a):\n from numpy import array\n a[0,0] = array([99,99])\n return a", numpy.array([[[1,2],[3,4]],[[5,6],[7,8]]]), np_change_arrayND_=[NDArray[int, :,:,:]])

    def test_ndim_(self):
        self.run_test("def np_ndim_(a): return a.ndim", numpy.array([[[1,2],[3,4]],[[5,6],[7,8]]]), np_ndim_=[NDArray[int, :,:,:]])

    def test_stride_(self):
        self.run_test("def np_stride_(a): return a.strides", numpy.array([[[1,2],[3,4]],[[5,6],[7,8]]]), np_stride_=[NDArray[int, :,:,:]])

    def test_size_(self):
        self.run_test("def np_size_(a): return a.size", numpy.array([[[1,2],[3,4]],[[5,6],[7,8]]]), np_size_=[NDArray[int, :,:,:]])

    def test_itemsize_(self):
        self.run_test("def np_itemsize_(a): return a.itemsize", numpy.array([[[1,2],[3,4]],[[5,6],[7,8]]]), np_itemsize_=[NDArray[int, :,:,:]])

    def test_nbytes_(self):
        self.run_test("def np_nbytes_(a): return a.nbytes", numpy.array([[[1,2],[3,4]],[[5,6],[7,8]]]), np_nbytes_=[NDArray[int, :,:,:]])

    def test_flat_(self):
        self.run_test("def np_flat_(a): return [i for i in a.flat]", numpy.array([[[1,2],[3,4]],[[5,6],[7,8]]]), np_flat_=[NDArray[int, :,:,:]])

    def test_sliced0(self):
        self.run_test("def np_sliced0(a): return a[2:12]", numpy.ones(20), np_sliced0=[NDArray[float, :]])

    def test_sliced1(self):
        self.run_test("def np_sliced1(a): return a[2:12:3]", numpy.ones(20), np_sliced1=[NDArray[float, :]])

    def test_sliced2(self):
        self.run_test("def np_sliced2(a): return -a[2:12:3]", numpy.ones(20), np_sliced2=[NDArray[float, :]])

    def test_sliced3(self):
        self.run_test("def np_sliced3(a): return a[1:11:3] -a[2:12:3]", numpy.ones(20), np_sliced3=[NDArray[float, :]])

    def test_sliced4(self):
        self.run_test("def np_sliced4(a): return a[1:11] -a[2:12]", numpy.ones(20), np_sliced4=[NDArray[float, :]])

    def test_sliced5(self):
        self.run_test("def np_sliced5(a): return (-a[1:11]) + 3*a[2:12]", numpy.ones(20), np_sliced5=[NDArray[float, :]])

    def test_sliced6(self):
        self.run_test("def np_sliced6(a): return a[3:4]", numpy.arange(12).reshape(6,2), np_sliced6=[NDArray[int, :,: ]])

    def test_sliced7(self):
        self.run_test("def np_sliced7(a): a[3:4] = 1 ; return a", numpy.arange(12).reshape(6,2), np_sliced7=[NDArray[int, :,: ]])

    def test_sliced8(self):
        self.run_test("def np_sliced8(a): a[1:2] = 1 ; return a", numpy.arange(12).reshape(3,2,2), np_sliced8=[NDArray[int, :,:,:]])

    def test_sliced9(self):
        self.run_test("def np_sliced9(a): from numpy import arange ; a[1:2] = arange(4.).reshape((1,2,2)) ; return a", numpy.arange(12.).reshape(3,2,2), np_sliced9=[NDArray[float, :,:,:]])

    def test_sliced10(self):
        self.run_test("def np_sliced10(a): from numpy import arange ; a[1:-1:2] = arange(4.).reshape((1,2,2)) ; return a", numpy.arange(12.).reshape(3,2,2), np_sliced10=[NDArray[float, :,:,:]])

    def test_sliced11(self):
        self.run_test("def np_sliced11(a): return a[1::-2]", numpy.arange(12).reshape(3,2,2), np_sliced11=[NDArray[int, :,:,:]])

    def test_sliced12(self):
        self.run_test("def np_sliced12(a): return a[1::-2]", numpy.arange(12), np_sliced12=[NDArray[int, :]])

    def test_sliced13(self):
        self.run_test("def np_sliced13(a): return a[3::-3]", numpy.arange(11), np_sliced13=[NDArray[int, :]])

    def test_newaxis0(self):
        self.run_test("def np_newaxis0(a): return a[None]",
                      numpy.ones(5), np_newaxis0=[NDArray[float, :]])

    def test_newaxis1(self):
        self.run_test("def np_newaxis1(a): from numpy import newaxis; return a[newaxis]",
                      numpy.ones(5), np_newaxis1=[NDArray[float, :]])

    def test_newaxis2(self):
        self.run_test("def np_newaxis2(a): from numpy import newaxis; return a[newaxis,:,newaxis]",
                      numpy.ones(5), np_newaxis2=[NDArray[float, :]])

    def test_newaxis3(self):
        self.run_test("def np_newaxis3(a): from numpy import newaxis; return a[:,newaxis]",
                      numpy.ones(5), np_newaxis3=[NDArray[float, :]])

    def test_newaxis4(self):
                      self.run_test("def np_newaxis4(a): from numpy import newaxis; return a[newaxis,:,:]",
                                    numpy.ones((2,3)), np_newaxis4=[NDArray[float, :,: ]])

    def test_newaxis5(self):
        self.run_test("def np_newaxis5(a): from numpy import newaxis; return a[:,newaxis,:]",
                      numpy.ones((2,3)), np_newaxis5=[NDArray[float, :,: ]])

    def test_newaxis6(self):
        self.run_test("def np_newaxis6(a): from numpy import newaxis; return a[:,:,newaxis]",
                      numpy.ones((2,3)), np_newaxis6=[NDArray[float, :,: ]])

    def test_newaxis7(self):
        self.run_test("def np_newaxis7(a): from numpy import newaxis; return a[newaxis,1:,newaxis,:1,newaxis]",
                      numpy.ones((2,3)), np_newaxis7=[NDArray[float, :,: ]])

    def test_newaxis8(self):
        code = '''
import numpy as np
def A(N):
    return B(N + 1)[0:-1]
def B(M):
    if M == 1: return np.ones(1, float)
    n = np.arange(0, M)
    return 0.5 - 0.5*np.cos(2.0*np.pi*n/(M-1))
def newaxis8(n):
    p = 0.5
    w = np.power(A(n),p)[:,None]
    return w'''
        self.run_test(code,
                      5,
                      newaxis8=[int])

    def test_gexpr_composition0(self):
        self.run_test("def gexpr_composition0(a): return a[:,:,:][1]",
                      numpy.arange(16).reshape(2,2,2,2),
                      gexpr_composition0=[NDArray[int, :,:,:, :]])

    def test_gexpr_composition1(self):
        self.run_test("def gexpr_composition1(a): return a[:,:,:][1,:]",
                      numpy.arange(16).reshape(2,2,2,2),
                      gexpr_composition1=[NDArray[int,:,:,:,:]])

    def test_gexpr_composition2(self):
        self.run_test("def gexpr_composition2(a): return a[:,:,:][1,:,:]",
                      numpy.arange(16).reshape(2,2,2,2),
                      gexpr_composition2=[NDArray[int,:,:,:,:]])

    def test_gexpr_composition3(self):
        self.run_test("def gexpr_composition3(a): return a[:,1,:][:,:,:]",
                      numpy.arange(16).reshape(2,2,2,2),
                      gexpr_composition3=[NDArray[int,:,:,:,:]])

    def test_gexpr_composition4(self):
        self.run_test("def gexpr_composition4(a): return a[:,:,1][:,:,:]",
                      numpy.arange(16).reshape(2,2,2,2),
                      gexpr_composition4=[NDArray[int,:,:,:,:]])

    def test_gexpr_composition5(self):
        self.run_test("def gexpr_composition5(a): return a[:,1,:][:,1,:]",
                      numpy.arange(16).reshape(2,2,2,2),
                      gexpr_composition5=[NDArray[int,:,:,:,:]])

    def test_gexpr_composition6(self):
        self.run_test("def gexpr_composition6(a): return a[:,:,1][:,1,:]",
                      numpy.arange(16).reshape(2,2,2,2),
                      gexpr_composition6=[NDArray[int,:,:,:,:]])

    def test_gexpr_composition7(self):
        self.run_test("def gexpr_composition7(a): return a[::2][::2]",
                      numpy.arange(16),
                      gexpr_composition7=[NDArray[int, :]])

    def test_gexpr_composition8(self):
        self.run_test("def gexpr_composition8(a): return a[1::2][2::2]",
                      numpy.arange(16),
                      gexpr_composition8=[NDArray[int, :]])

    def test_gexpr_composition9(self):
        self.run_test("def gexpr_composition9(a): return a[:1:2][:2:2]",
                      numpy.arange(16),
                      gexpr_composition9=[NDArray[int, :]])

    @unittest.skip("Unknown slice combination")
    def test_gexpr_composition10(self):
        self.run_test("def gexpr_composition10(a): return a[:-1:2][:-2:2]",
                      numpy.arange(16),
                      gexpr_composition10=[NDArray[int, :]])

    @unittest.skip("Unknown slice combination")
    def test_gexpr_composition11(self):
        self.run_test("def gexpr_composition11(a): return a[::2][::-2]",
                      numpy.arange(17),
                      gexpr_composition11=[NDArray[int, :]])

    @unittest.skip("Unknown slice combination")
    def test_gexpr_composition12(self):
        self.run_test("def gexpr_composition12(a): return a[::-2][::-2]",
                      numpy.arange(13),
                      gexpr_composition12=[NDArray[int, :]])

    def test_gexpr_composition13(self):
        self.run_test("def gexpr_composition13(a): return a[::-3][::2]",
                      numpy.arange(17),
                      gexpr_composition13=[NDArray[int, :]])

    def test_gexpr_composition14(self):
        self.run_test("def gexpr_composition14(a): return a[:,::2][:,1,::-2]",
                      numpy.arange(16).reshape(1,4,4),
                      gexpr_composition14=[NDArray[int, :,:,:]])

    def test_gexpr_composition15(self):
        self.run_test("def gexpr_composition15(a): return a[:,1,1:-1][:,:-1]",
                      numpy.arange(16).reshape(1,4,4),
                      gexpr_composition15=[NDArray[int, :,:,:]])

    def test_gexpr_composition16(self):
        self.run_test("def gexpr_composition16(a): return a[:,None][:,:,None]",
                      numpy.arange(16).reshape(4,4),
                      gexpr_composition16=[NDArray[int, :,:]])

    def test_gexpr_composition17(self):
        self.run_test("def gexpr_composition17(a): return a[:,None][:,None]",
                      numpy.arange(16).reshape(1,4,4),
                      gexpr_composition17=[NDArray[int, :,:,:]])

    def test_gexpr_composition18(self):
        self.run_test("def gexpr_composition18(a): return a[0,:,None][:,None]",
                      numpy.arange(24).reshape(2,3,4),
                      gexpr_composition18=[NDArray[int, :,:,:]])

    def test_gexpr_composition19(self):
        self.run_test("def gexpr_composition19(a): return a[0:3][1:2]",
                      numpy.arange(24),
                      gexpr_composition19=[NDArray[int, :]])


    def test_gexpr_copy0(self):
        self.run_test("def gexpr_copy0(a,b): a[:,0] = b[:,0]; return a",
                      numpy.arange(16).reshape(8,2), numpy.arange(16).reshape(8,2),
                      gexpr_copy0=[NDArray[int, :,: ], NDArray[int, :,: ]])

    def test_gexpr_copy1(self):
        self.run_test("def gexpr_copy1(a): a[:] = a[::-1]; return a",
                      numpy.arange(16),
                      gexpr_copy1=[NDArray[int, :]])

    def test_ndarray_iter0(self):
        self.run_test("def ndarray_iter0(a): return list(map(str, a))",
                      numpy.arange(16),
                      ndarray_iter0=[NDArray[int, :]])

    def test_ndarray_iter1(self):
        self.run_test("""
                      def ndarray_iter1(a):
                        s = 0
                        for v in a:
                            s *= v
                        return s""",
                      numpy.arange(16),
                      ndarray_iter1=[NDArray[int, :]])

    def test_ndarray_str_dtype0(self):
        self.run_test("def ndarray_str_dtype0(a): return str(a.dtype)",
                      numpy.arange(16),
                      ndarray_str_dtype0=[NDArray[int, :]])

    def test_ndarray_str_dtype1(self):
        self.run_test("def ndarray_str_dtype1(a): return str(a.dtype)",
                      numpy.arange(16.),
                      ndarray_str_dtype1=[NDArray[float, :]])

    def test_ndarray_fancy_indexing0(self):
        self.run_test("def ndarray_fancy_indexing0(a,b): return a[b]",
                numpy.arange(8.),
                numpy.arange(8),
                ndarray_fancy_indexing0=[NDArray[float, :], NDArray[int, :]])

    def test_ndarray_fancy_indexing1(self):
        self.run_test("def ndarray_fancy_indexing1(a,b): return a[b]",
                numpy.arange(8.).reshape(4,2),
                numpy.array([0,1], dtype=int),
                ndarray_fancy_indexing1=[NDArray[float, :, :], NDArray[int, :]])

    def test_ndarray_fancy_indexing2(self):
        self.run_test("def ndarray_fancy_indexing2(a,b): return a[b]",
                numpy.arange(8.).reshape(4,2),
                numpy.array([3,2,1,0, 0], dtype=int),
                ndarray_fancy_indexing2=[NDArray[float, :, :], NDArray[int, :]])

    def test_ndarray_fancy_indexing3(self):
        self.run_test("def ndarray_fancy_indexing3(a,b): return a[b]",
                numpy.arange(8.).reshape(4,2),
                numpy.array([3,2,1,0], dtype=int),
                ndarray_fancy_indexing3=[NDArray[float, :, :], NDArray[int, :]])

    def test_ndarray_ubyte(self):
        self.run_test("def ndarray_ubyte(n): import numpy; return numpy.arange(0, n, 1, dtype=numpy.ubyte)",
                4,
                ndarray_ubyte=[int])

    def test_ndarray_byte(self):
        self.run_test("def ndarray_byte(n): import numpy; return numpy.arange(-n, n, 1, dtype=numpy.byte)",
                4,
                ndarray_byte=[int])

    def test_ndarray_ushort(self):
        self.run_test("def ndarray_ushort(n): import numpy; return numpy.arange(0, n, 1, dtype=numpy.ushort)",
                4,
                ndarray_ushort=[int])

    def test_ndarray_short(self):
        self.run_test("def ndarray_short(n): import numpy; return numpy.arange(-n, n, 1, dtype=numpy.short)",
                4,
                ndarray_short=[int])

    def test_ndarray_1d_index(self):
        self.run_test(
            'def ndarray_1d_index(a): return a[1], a[-1]',
            numpy.arange(30).reshape((2,3,5)),
            ndarray_1d_index=[NDArray[int, :, :,:]])

    def test_ndarray_2d_index(self):
        self.run_test(
            'def ndarray_2d_index(a): return a[0,1], a[0, -1]',
            numpy.arange(30).reshape((2,3,5)),
            ndarray_2d_index=[NDArray[int, :, :,:]])

    def test_ndarray_3d_index(self):
        self.run_test(
            'def ndarray_3d_index(a): return a[0, 1, 2], a[0, -1, -2]',
            numpy.arange(30).reshape((2,3,5)),
            ndarray_3d_index=[NDArray[int, :, :,:]])

    def test_numpy_iexpr_1d_index(self):
        self.run_test(
            'def numpy_iexpr_1d_index(A): a = A[0]; return a[1], a[-1]',
            numpy.arange(30).reshape((1, 2,3,5)),
            numpy_iexpr_1d_index=[NDArray[int, :, :,:, :]])

    def test_numpy_iexpr_2d_index(self):
        self.run_test(
            'def numpy_iexpr_2d_index(A): a = A[0]; return a[0,1], a[0, -1]',
            numpy.arange(30).reshape((1, 2,3,5)),
            numpy_iexpr_2d_index=[NDArray[int, :, :,:, :]])

    def test_numpy_iexpr_3d_index(self):
        self.run_test(
            'def numpy_iexpr_3d_index(A): a = A[0]; return a[0, 1, 2], a[0, -1, -2]',
            numpy.arange(30).reshape((1, 2,3,5)),
            numpy_iexpr_3d_index=[NDArray[int, :, :,:, :]])


    def test_numpy_indexing_ex0(self):
        self.run_test(
            'def numpy_indexing_ex0(x, y): return x[y]',
            numpy.array([1,2,3,4]),
            (0,),
            numpy_indexing_ex0=[NDArray[int, :], Tuple[int]])

    def test_numpy_indexing_ex1(self):
        self.run_test(
            'def numpy_indexing_ex1(x, y): return x[y]',
            numpy.array([[1,2],[3,4]]),
            (0, 1),
            numpy_indexing_ex1=[NDArray[int, :, :], Tuple[int, int]])

    def test_numpy_indexing_ex2(self):
        self.run_test(
            'def numpy_indexing_ex2(x, y): return x[y, :]',
            numpy.array([[1,2],[3,4]]),
            0,
            numpy_indexing_ex2=[NDArray[int, :, :], int])

    def test_numpy_indexing_ex3(self):
        self.run_test(
            'def numpy_indexing_ex3(x, y): return x[y]',
            numpy.array([[1,2],[3,4]]),
            [1],
            numpy_indexing_ex3=[NDArray[int, :, :], List[int]])

    def test_numpy_indexing_ex4(self):
        self.run_test(
            'def numpy_indexing_ex4(x, y): return x[:,:][y]',
            numpy.array([[1,2],[3,4]]),
            [1],
            numpy_indexing_ex4=[NDArray[int, :, :], List[int]])

    def test_numpy_indexing_ex10(self):
        self.run_test(
            'def numpy_indexing_ex10(x, y): return (2*x)[y]',
            numpy.array([1,2,3,4]),
            (0,),
            numpy_indexing_ex10=[NDArray[int, :], Tuple[int]])

    def test_numpy_indexing_ex11(self):
        self.run_test(
            'def numpy_indexing_ex11(x, y): return (2*x)[y]',
            numpy.array([[1,2],[3,4]]),
            (0, 1),
            numpy_indexing_ex11=[NDArray[int, :, :], Tuple[int, int]])

    def test_numpy_indexing_ex12(self):
        self.run_test(
            'def numpy_indexing_ex12(x, y): return (2*x)[y, :]',
            numpy.array([[1,2],[3,4]]),
            0,
            numpy_indexing_ex12=[NDArray[int, :, :], int])

    def test_numpy_indexing_ex13(self):
        self.run_test(
            'def numpy_indexing_ex13(x, y): return (2*x)[y]',
            numpy.array([[1,2],[3,4]]),
            [1],
            numpy_indexing_ex13=[NDArray[int, :, :], List[int]])

    def test_numpy_indexing_ex14(self):
        self.run_test(
            'def numpy_indexing_ex14(x, y): return (2*x)[:,:][y]',
            numpy.array([[1,2],[3,4]]),
            [1],
            numpy_indexing_ex14=[NDArray[int, :, :], List[int]])

    def test_numpy_expr_combiner(self):
        code = '''
            import numpy as np

            def abcd(x, y):
                return np.linspace(0, x, y)

            def ops(a, b, o):
                return 4.0 * np.pi / a * np.sin(0.5 * np.arctan(o / b))

            def efgh(x, y):
                a = abcd(x, y)
                if x == 2:
                    c = ops(2.0, 3.0, a * 3.0) # Comment here
                    #c = a # decoment here
                else:
                    c = a # And here
                    #c = ops(2.0, 3.0, a * 3.0) # and here
                return c[:-1]

            def numpy_expr_combiner(x, y):
                b = efgh(x, y)
                return b'''
        self.run_test(code,
                      10, 10,
                      numpy_expr_combiner=[int, int])

    def test_bool_conversion_0(self):
        code = 'def bool_conversion_0(x): return bool(x)'
        self.run_test(code,
                      numpy.array([[1]]),
                      bool_conversion_0=[NDArray[int, :,:]])

    def test_bool_conversion_1(self):
        code = 'def bool_conversion_1(x): return bool(x > 1)'
        self.run_test(code,
                      numpy.array([1]),
                      bool_conversion_1=[NDArray[int, :]])

    def test_bool_conversion_2(self):
        code = 'def bool_conversion_2(x): return bool(x[1:,1,1:])'
        self.run_test(code,
                      numpy.array([[[1,1],[1,1]],
                                   [[1,0],[1,1]]]
                                   ),
                                   bool_conversion_2=[NDArray[int, :,:,:]])

    def test_bool_conversion_3(self):
        code = 'def bool_conversion_3(x): return bool(x[1])'
        self.run_test(code,
                      numpy.array([[1],[0]]),
                      bool_conversion_3=[NDArray[int, :,:]])

    def test_bool_conversion_4(self):
        code = 'def bool_conversion_4(x): return bool(x.T)'
        self.run_test(code,
                      numpy.array([[1]]),
                      bool_conversion_4=[NDArray[int, :,:]])

    def test_complex_conversion0(self):
        code = '''
import numpy as np

def test1(input):
    fftSize = 1024
    NN = 10
    MM = fftSize // 2+1
    Out = np.empty((MM, NN), dtype=np.complex64)
    for frm in range(NN):
        X = np.fft.rfft(input[0:fftSize],fftSize)
        Out[:,frm] = X
    return Out

def complex_conversion0(x):
    return test1(x)
'''
        self.run_test(code, numpy.array([10.]*10),
                      complex_conversion0=[NDArray[float, :]])


    def test_numpy_lazy_gexpr(self):
        code = '''
            import numpy as np

            def efgh2(s, m, x, y):
                if x == 0: # Without the if it doesn't compile
                    return np.zeros(10)[:-1]
                return np.zeros(10)[:-1]

            def numpy_lazy_gexpr(s, m, x, y):
                if m == 'a':
                    b = efgh2(s, m, x, y + 1)
                else:
                    b = efgh2(s, m, x, y)
                return b'''
        self.run_test(code,
                      10, "b", 10, 10,
                      numpy_lazy_gexpr=[int, str, int, int])

    def test_numpy_lazy_gexpr2(self):
        code = '''
            import numpy as np
            def numpy_lazy_gexpr2(c, y):
                z = [0] * 2
                if c:
                    x = y[1:3]
                z[0] = x[0]
                z[1] = x[1]
                return z'''
        self.run_test(code,
                      1, numpy.array([1,2,3,4]),
                      numpy_lazy_gexpr2=[int, NDArray[int, :]])

    def test_fexpr0(self):
        code = '''
            import numpy as np
            def test_fexpr0(peaks):
                peaks = peaks[peaks>30] # <- does not compiles
                lastBin = 0
                for peak in peaks:
                    lastBin = peak
                return lastBin'''
        self.run_test(code, numpy.array([1,32,33,4]), test_fexpr0=[NDArray[int, :]])

    def test_fexpr1(self):
        code = '''
            import numpy as np
            def yy(x):
                a = np.zeros(x, np.float32)
                return a[:-1]

            def test_fexpr1(x):
                c = yy(x)
                d = yy(x)
                c[d == 0] = np.nan
                return c'''
        self.run_test(code, 10, test_fexpr1=[int])


    def test_vexpr0(self):
        code = '''
            import numpy as np
            def vexpr0(a, b=None):
                if b is None:
                    assert len(a) > 0
                    b = np.copy(a[0])
                    a = a[1:]
                else:
                    b = np.copy(b)
                m = b >= 0
                for array in a:
                    b[m] *= array[m]
                return b'''
        self.run_test(code,
            [numpy.arange(10, dtype=numpy.int32).reshape(5,2)],
            2 * numpy.arange(10, dtype=numpy.int32).reshape(5,2),
            vexpr0=[List[NDArray[numpy.int32,:,:]], NDArray[numpy.int32,:,:]])

    def test_array_of_pshape(self):
        code = 'def array_of_pshape(x): from numpy import array; return array(x[None].shape)'
        self.run_test(code, numpy.arange(10), array_of_pshape=[NDArray[int,:]])

    def test_vexpr_of_texpr(self):
        code = '''
        import numpy as np
        def apply_mask(mat, mask):
            assert mask.shape == mat.shape
            mat[mask == False] = np.nan
            return mat

        def vexpr_of_texpr(a, b):
            return apply_mask(a.T, b), apply_mask(a, b.T), apply_mask(a.T, b.T)'''
        self.run_test(code,
                      numpy.arange(4., dtype=numpy.float32).reshape(2,2),
                      numpy.array([[False,True],[True, False]]),
                      vexpr_of_texpr=[NDArray[numpy.float32,:,:], NDArray[numpy.bool,:,:]])

    def test_indexing_through_int8(self):
        code = '''
            def indexing_through_int8(x):
                return x[x[0,0],x[0,1]]'''
        self.run_test(code,
                      numpy.arange(10, dtype=numpy.uint8).reshape(5,2),
                      indexing_through_int8=[NDArray[numpy.uint8,:,:]])

    def test_indexing_through_byte(self):
        code = '''
            def indexing_through_byte(x):
                return x[x[0,0],x[0,1]]'''
        self.run_test(code,
                      numpy.arange(10, dtype=numpy.byte).reshape(5,2),
                      indexing_through_byte=[NDArray[numpy.byte,:,:]])

    def test_indexing_through_uint8(self):
        code = '''
            def indexing_through_uint8(x):
                import numpy as np
                return x[np.uint8(2), np.uint8(1)]'''
        self.run_test(code,
                      numpy.arange(9).reshape(3,3),
                      indexing_through_uint8=[NDArray[int,:,:]])

    def test_complex_scalar_broadcast(self):
        self.run_test('def complex_scalar_broadcast(a): return (a**2 * (1 + a) + 2) / 5.',
                      numpy.ones((10,10), dtype=complex),
                      complex_scalar_broadcast=[NDArray[complex, :, :]])


    @unittest.skipIf(not has_float128, "not float128")
    def test_float128_0(self):
        self.run_test('def float128_0(x): return x, x **2',
                      numpy.float128(numpy.finfo(numpy.float64).max),
                      float128_0=[numpy.float128])

    @unittest.skipIf(not has_float128, "not float128")
    def test_float128_1(self):
        self.run_test('def float128_1(x): return x, x **2',
                      numpy.ones((10,10), dtype=numpy.float128),
                      float128_1=[NDArray[numpy.float128,:, :]])

    @unittest.skipIf(not has_float128, "not float128")
    def test_float128_2(self):
        self.run_test('def float128_2(x): from numpy import ones, float128; return ones(x,dtype=float128)',
                      3,
                      float128_2=[int])

    def test_texpr_expr_combined(self):
        self.run_test("def texpr_expr_combined(x, y):\n if x: return y.T\n else: return y * 2",
                      1,
                      numpy.arange(10).reshape(5, 2),
                      texpr_expr_combined=[int, NDArray[int,:,:]])

    def test_built_slice_indexing(self):
        self.run_test('''
            def built_slice_indexing(x,n,axis,val=0.):
                import numpy as np
                y = np.roll(x,n,axis)
                S = [slice(None,None)]*x.ndim
                if n>0: S[axis] = slice(0, n)
                elif n<0: S[axis] = slice(n, None)
                if n: y[tuple(S)] = val
                return y''',
            numpy.array([-1.2, 1, 1.2]),
            1,
            0,
            5.,
            built_slice_indexing=[NDArray[float, :], int, int, float])

    def test_dtype_type(self):
        self.run_test('''
            def dtype_type(x):
                import numpy as np
                c = np.complex64(x)
                f = np.float32(x)
                u = np.uint8(x)
                return c.dtype.type(1), f.dtype.type(2), u.dtype.type(3)''',
            2,
            dtype_type=[int])

    def test_transposed_slice_assign0(self):
                self.run_test("""def transposed_slice_assign0(shape):
                                  import numpy as np
                                  xx = np.empty(shape, dtype=int)
                                  xx.T[:] = np.arange(0, shape[0], 1, dtype=int)
                                  return xx""",
                              (3, 5),
                              transposed_slice_assign0=[Tuple[int, int]])

    def test_transposed_slice_assign1(self):
                self.run_test("""def transposed_slice_assign1(shape):
                                  import numpy as np
                                  xx = np.ones(shape, dtype=int)
                                  xx.T[:] = 3
                                  return xx""",
                              (3, 5),
                              transposed_slice_assign1=[Tuple[int, int]])

    def test_transposed_slice_assign2(self):
                self.run_test("""def transposed_slice_assign2(shape):
                                  import numpy as np
                                  xx = np.ones(shape, dtype=int)
                                  xx.T[:1] = 3
                                  return xx""",
                              (10, 20),
                              transposed_slice_assign2=[Tuple[int, int]])

    def test_transposed_slice_assign3(self):
                self.run_test("""def transposed_slice_assign3(shape):
                                  import numpy as np
                                  xx = np.ones(shape, dtype=int).T
                                  xx[:1, :2] = 3
                                  return xx""",
                              (10, 20),
                              transposed_slice_assign3=[Tuple[int, int]])

    def test_slice_through_list0(self):
                self.run_test("""def slice_through_list0(shape):
                                  import numpy as np
                                  xx = np.ones(shape, dtype=int)
                                  return xx[[1,3,5],1]""",
                              (10, 20),
                              slice_through_list0=[Tuple[int, int]])

    def test_slice_through_list1(self):
                self.run_test("""def slice_through_list1(shape):
                                  import numpy as np
                                  xx = np.ones(shape, dtype=int)
                                  return xx[[1,3,5],1:]""",
                              (10, 20),
                              slice_through_list1=[Tuple[int, int]])

    def test_slice_through_list2(self):
                self.run_test("""def slice_through_list2(arr):
                                  import numpy as np;
                                  return arr[np.array([1,3,5]),1:]""",
                              numpy.ones((10, 20)),
                              slice_through_list2=[NDArray[float, :,:]])

    def test_slice_through_list3(self):
                self.run_test("""def slice_through_list3(arr):
                                  import numpy as np;
                                  return arr[[1,3,5],1:]""",
                              numpy.ones((10, 20)),
                              slice_through_list3=[NDArray[float, :,:]])

    def test_slice_through_list4(self):
                self.run_test("""def slice_through_list4(arr):
                                  import numpy as np;
                                  return arr[(1,3,5),1:]""",
                              numpy.ones((10, 20)),
                              slice_through_list4=[NDArray[float, :,:]])


    def test_transposed_array(self):
                self.run_test("""def transposed_array(shape):
                                  import numpy as np
                                  xx = np.ones(shape, dtype=int)
                                  return xx.T""",
                              (3, 5),
                              transposed_array=[Tuple[int, int]])

    def assign_transposed(self, last):
        params = [numpy.arange(30.).reshape(5,6), 3, 1, 1]
        code = '''
            def helper(signal, N, A):
                return (signal[N:N-A:-1],
                        signal[N:N+A],
                        signal[N-A:N+A],)

            def assign_transposed(signal, N, A):
                return (helper(signal, N, A),
                        helper(signal[:,:], N, A),
                        helper(signal.T, N, A),)
            '''
        self.run_test(code, *params, assign_transposed=[NDArray[float,:,:], int, int, int])

    def test_input_asarray(self):
        code = 'def input_asarray(x): return x'
        self.run_test(code, numpy.arange(12.).reshape((1, 3,4))[:, :2, :],
                      input_asarray=[NDArray[float,:,:,:]])

    def test_input_strided_array(self):
        code = 'def input_strided_array(x): return x'
        with self.assertRaises(TypeError):
            self.run_test(code, numpy.arange(12.)[::2],
                          input_strided_array=[NDArray[float,:]])

    def test_input_unit_strided_array(self):
        code = 'def input_unit_strided_array(x): return x'
        self.run_test(code, numpy.arange(12.)[::1],
                      input_unit_strided_array=[NDArray[float,:]])

    def test_input_empty_strided_array(self):
        code = 'def input_empty_strided_array(x): return x'
        self.run_test(code, numpy.random.random((0, 2, 3))[:,:,:2],
                      input_empty_strided_array=[NDArray[float,:,:,:]])

    def test_hanning(self):
        code = '''
import numpy as np
def helper(M):
    if M < 1:
        return np.array([])
    if M == 1:
        return np.ones(1, float)
    n = np.arange(0,float(M))
    return 0.5 - 0.5*np.cos(2.0*np.pi*n/(M-1))
def hanning(M):
    return helper(M * 0), helper(M * 1), helper(M * 4)'''
        self.run_test(code, 1, hanning=[int])

    def test_ones_on_updated_shape(self):
        code = '''
            import numpy as np
            def ones_on_updated_shape(array,n,axis,padVal):
                shape = list(array.shape)
                shape[axis]=n
                toPad = padVal*np.ones(shape,dtype=array.dtype)
                return np.concatenate((np.diff(array, n, axis=axis), toPad), axis=axis)'''
        self.run_test(code, numpy.arange(10.).reshape(5, 2), 1, 0, 3., ones_on_updated_shape=[NDArray[float,:,:], int, int, float])

    def test_numpy_vexpr_static_shape(self):
        code = '''
            import numpy as np
            def numpy_vexpr_static_shape(A):
                x_transFrames = np.array([3,6,20,40,70,245])
                x_transScores = A[x_transFrames]
                x_transList = np.array([0,2,3,4])
                B = x_transScores[x_transList]
                return  x_transFrames[x_transList].astype(int),B'''
        self.run_test(code, numpy.arange(300.), numpy_vexpr_static_shape=[NDArray[float, :]])

    def test_subscripting_slice_array_transpose(self):
        code = 'def subscripting_slice_array_transpose(x): return x.T[(slice(0,1),slice(0,1))]'
        self.run_test(code,
                      numpy.arange(200.).reshape(10, 20),
                      subscripting_slice_array_transpose=[NDArray[float, :, :]])

    def test_combiner_0(self):
        code = '''
import numpy as np
def test_combiner_0(X):
    O=test1(X[:,0], False)
    P=test1(X[:,0], True)   # <- Ask to concatenate
    return O,P

def test1(X,A):
    N = 20
    if A:
        X = np.concatenate((np.zeros(N),X))
    return X'''
        self.run_test(code, numpy.ones((10,10)), test_combiner_0=[NDArray[float,:,:]])

    def test_combiner_1(self):
        code = '''
import numpy as np
def test_combiner_1(X):
    O=test1(X[0], False)
    P=test1(X[1], True)   # <- Ask to concatenate
    return O,P

def test1(X,A):
    N = 20
    if A:
        X = np.concatenate((np.zeros((N)),X))
    return X'''
        self.run_test(code, numpy.ones((10,10)), test_combiner_1=[NDArray[float,:,:]])

    def test_combiner_2(self):
        code = '''
import numpy as np
def test_combiner_2(X):
    O=test1(X[X==1], False)
    P=test1(X[X==1], True)   # <- Ask to concatenate
    return O,P

def test1(X,A):
    N = 20
    if A:
        X = np.concatenate((np.zeros((N)),X))
    return X'''
        self.run_test(code, numpy.ones((10)), test_combiner_2=[NDArray[float,:]])



from distutils.core import setup
from Cython.Build import cythonize

setup(
    name = "indexing",
    ext_modules = cythonize('indexing.pyx'),
    script_name = 'setup.py',
    script_args = ['build_ext', '--inplace']
)
import indexing
import numpy as np
print(indexing.indexing())




#from http://rosettacode.org/wiki/Pi#Python
#pythran export test()
#runas test()
#FIXME unittest.skip

def calcPi():
    q, r, t, k, n, l = 1, 0, 1, 1, 3, 3
    while True:
        if 4*q+r-t < n*t:
            yield n
            nr = 10*(r-n*t)
            n  = ((10*(3*q+r))//t)-10*n
            q  *= 10
            r  = nr
        else:
            nr = (2*q+r)*l
            nn = (q*(7*k)+2+(r*l))//(t*l)
            q  *= k
            t  *= l
            l  += 2
            k += 1
            n  = nn
            r  = nr

def test():
    pi_digits = calcPi()
    res = list()
    for i, d in enumerate(pi_digits):
        res.append(str(d))
        if i>50:
            return res



#from http://rosettacode.org/wiki/Zeckendorf_number_representation#Python
#pythran export test(int)
#pythran export z(int)
#pythran export zeckendorf(int)
#runas test(20)
#runas ['%3i: %8s' % (i, ''.join(str(d) for d in zeckendorf(i))) for i in range(21)]
#runas ['%3i: %8s' % (i, ''.join(str(d) for d in z(i))) for i in range(21)]

def fib():
    memo = [1, 2]
    while True:
        memo.append(sum(memo))
        yield memo.pop(0)

def sequence_down_from_n(n, seq_generator):
    seq = []
    for s in seq_generator():
        seq.append(s)
        if s >= n: break
    return seq[::-1]

def zeckendorf(n):
    if n == 0: return [0]
    seq = sequence_down_from_n(n, fib)
    digits, nleft = [], n
    for s in seq:
        if s <= nleft:
            digits.append(1)
            nleft -= s
        else:
            digits.append(0)
    assert nleft == 0, 'Check all of n is accounted for'
    assert sum(x*y for x,y in zip(digits, seq)) == n, 'Assert digits are correct'
    while digits[0] == 0:
        # Remove any zeroes padding L.H.S.
        digits.pop(0)
    return digits

def z(n):
    if n == 0 : return [0]
    fib = [2,1]
    while fib[0] < n: fib[0:0] = [sum(fib[:2])]
    dig = []
    for f in fib:
        if f <= n:
            dig, n = dig + [1], n - f
        else:
            dig += [0]
    return dig if dig[0] else dig[1:]

def test(n):
    return sequence_down_from_n(n, fib)




#from http://jakevdp.github.com/blog/2012/08/24/numba-vs-cython/
#runas import numpy as np ; X = np.linspace(0,10,20000).reshape(200,100) ; pairwise(X)
#bench import numpy as np ; X = np.linspace(0,10,10000).reshape(100,100) ; pairwise(X)
#pythran export pairwise(float [][])

import numpy as np
def pairwise(X):
    M, N = X.shape
    D = np.empty((M,M))
    for i in range(M):
        for j in range(M):
            d = 0.0
            for k in range(N):
                tmp = X[i,k] - X[j,k]
                d += tmp * tmp
            D[i,j] = np.sqrt(d)
    return D



#from http://rosettacode.org/wiki/Polynomial_long_division#Python
from math import fabs

#pythran export poly_div(int list, int list)
#runas poly_div([-42, 0, -12, 1], [-3, 1, 0, 0])

def degree(poly):
    while poly and poly[-1] == 0:
        poly.pop()   # normalize
    return len(poly)-1

def poly_div(N, D):
    dD = degree(D)
    dN = degree(N)
    if dD < 0: raise ZeroDivisionError
    if dN >= dD:
        q = [0] * dN
        while dN >= dD:
            d = [0]*(dN - dD) + D
            mult = q[dN - dD] = N[-1] / float(d[-1])
            d = [coeff*mult for coeff in d]
            N = [fabs ( coeffN - coeffd ) for coeffN, coeffd in zip(N, d)]
            dN = degree(N)
        r = N
    else:
        q = [0]
        r = N
    return q, r



#pythran export fft(complex [])
#runas from numpy import ones ; a = ones(2**10, dtype=complex) ; fft(a)
#bench from numpy import ones ; a = ones(2**14, dtype=complex) ; fft(a)

import math, numpy as np

def fft(x):
   N = x.shape[0]
   if N == 1:
       return np.array(x)
   e=fft(x[::2])
   o=fft(x[1::2])
   M=N//2
   l=[ e[k] + o[k]*math.e**(-2j*math.pi*k/N) for k in range(M) ]
   r=[ e[k] - o[k]*math.e**(-2j*math.pi*k/N) for k in range(M) ]
   return np.array(l+r)




#from http://rosettacode.org/wiki/Array_concatenation#Python
#pythran export array_concatenation(int list, int list, int list, int list, int list)
#runas arr1 = [1, 2, 3]; arr2 = [4, 5, 6]; arr3 = [7, 8, 9]; arr5 = [4, 5, 6]; arr6 = [7, 8, 9]; array_concatenation(arr1, arr2, arr3, arr5, arr6)

def array_concatenation(arr1, arr2, arr3, arr5, arr6):
    arr4 = arr1 + arr2
    assert arr4 == [1, 2, 3, 4, 5, 6]
    arr4.extend(arr3)
    assert arr4 == [1, 2, 3, 4, 5, 6, 7, 8, 9]

    arr6 += arr5
    assert arr6 == [7, 8, 9, 4, 5, 6]



import numpy as np

# pythran export _dilate(uint8[:, :], uint8[:, :], uint8[:, :], int8, int8)
# pythran export _dilate(uint8[:, :], uint8[:, :], uint8[:, :], int8)
# pythran export _dilate(uint8[:, :], uint8[:, :], uint8[:, :])
# pythran export _dilate(uint8[:, :], uint8[:, :])


def _dilate(image, selem, out=None, shift_x=0, shift_y=0):
    """Return greyscale morphological dilation of an image.

    Morphological dilation sets a pixel at (i,j) to the maximum over all pixels
    in the neighborhood centered at (i,j). Dilation enlarges bright regions
    and shrinks dark regions.

    Parameters
    ----------

    image : ndarray
        Image array.
    selem : ndarray
        The neighborhood expressed as a 2-D array of 1's and 0's.
    out : ndarray
        The array to store the result of the morphology. If None, is
        passed, a new array will be allocated.
    shift_x, shift_y : bool
        shift structuring element about center point. This only affects
        eccentric structuring elements (i.e. selem with even numbered sides).

    Returns
    -------
    dilated : uint8 array
        The result of the morphological dilation.
    """

    rows = image.shape[0]
    cols = image.shape[1]
    srows = selem.shape[0]
    scols = selem.shape[1]

    centre_r = int(selem.shape[0] / 2) - shift_y
    centre_c = int(selem.shape[1] / 2) - shift_x

    image = np.ascontiguousarray(image)
    if out is None:
        out = np.zeros((rows, cols), dtype=np.uint8)

    selem_num = np.sum(np.asarray(selem) != 0)
    sr = np.empty(selem_num, dtype=np.intp)
    sc = np.empty(selem_num, dtype=np.intp)

    s = 0
    for r in range(srows):
        for c in range(scols):
            if selem[r, c] != 0:
                sr[s] = r - centre_r
                sc[s] = c - centre_c
                s += 1

    for r in range(rows):
        for c in range(cols):
            local_max = 0
            for s in range(selem_num):
                rr = r + sr[s]
                cc = c + sc[s]
                if 0 <= rr < rows and 0 <= cc < cols:
                    value = image[rr, cc]
                    if value > local_max:
                        local_max = value

            out[r, c] = local_max

    return np.asarray(out)


# pythran export _erode(uint8[:, :], uint8[:, :], uint8[:, :], int8, int8)
# pythran export _erode(uint8[:, :], uint8[:, :], uint8[:, :], int8)
# pythran export _erode(uint8[:, :], uint8[:, :], uint8[:, :])
# pythran export _erode(uint8[:, :], uint8[:, :])


def _erode(image, selem, out=None, shift_x=0, shift_y=0):
    """Return greyscale morphological erosion of an image.

    Morphological erosion sets a pixel at (i,j) to the minimum over all pixels
    in the neighborhood centered at (i,j). Erosion shrinks bright regions and
    enlarges dark regions.

    Parameters
    ----------
    image : ndarray
        Image array.
    selem : ndarray
        The neighborhood expressed as a 2-D array of 1's and 0's.
    out : ndarray
        The array to store the result of the morphology. If None is
        passed, a new array will be allocated.
    shift_x, shift_y : bool
        shift structuring element about center point. This only affects
        eccentric structuring elements (i.e. selem with even numbered sides).

    Returns
    -------
    eroded : uint8 array
        The result of the morphological erosion.
    """

    rows = image.shape[0]
    cols = image.shape[1]
    srows = selem.shape[0]
    scols = selem.shape[1]

    centre_r = int(selem.shape[0] / 2) - shift_y
    centre_c = int(selem.shape[1] / 2) - shift_x

    image = np.ascontiguousarray(image)
    if out is None:
        out = np.zeros((rows, cols), dtype=np.uint8)

    selem_num = np.sum(np.asarray(selem) != 0)
    sr = np.empty(selem_num, dtype=np.intp)
    sc = np.empty(selem_num, dtype=np.intp)

    s = 0
    for r in range(srows):
        for c in range(scols):
            if selem[r, c] != 0:
                sr[s] = r - centre_r
                sc[s] = c - centre_c
                s += 1

    for r in range(rows):
        for c in range(cols):
            local_min = 255
            for s in range(selem_num):
                rr = r + sr[s]
                cc = c + sc[s]
                if 0 <= rr < rows and 0 <= cc < cols:
                    value = image[rr, cc]
                    if value < local_min:
                        local_min = value

            out[r, c] = local_min

    return np.asarray(out)



def do_some_work():
    import math
    sum = 0.
    for i in range(1000):
        sum+=math.sqrt(i)

def omp_for_private():
    sum = 0
    sum0 = 0
    LOOPCOUNT = 1000
    if 'omp parallel':
        sum1 = 0
        'omp for private(sum0) schedule(static,1)'
        for i in range(1, LOOPCOUNT+1):
            sum0 = sum1
            'omp flush'
            sum0 += i
            do_some_work()
            'omp flush'
            sum1 = sum0
        'omp critical'
        sum += sum1
    known_sum = (LOOPCOUNT * (LOOPCOUNT + 1)) / 2
    return known_sum == sum



#pythran export CAXPY(int, complex, complex list, int, complex list, int)
#runas CAXPY(2,complex(1.1,2.3),[complex(1,2),complex(2,3),complex(3,4),complex(5,6)],2,[complex(3,4),complex(1,2),complex(2,3),complex(5,6)],3)
def CAXPY(N,CA,CX,INCX,CY,INCY):
#  Purpose
#  =======
#
#     CAXPY constant times a vector plus a vector.
#
#  Further Details
#  ===============
#
#     jack dongarra, linpack, 3/11/78.
#     modified 12/3/93, array(1) declarations changed to array(#)
#
#  =====================================================================
#
        if N <= 0:
                return
        if (abs(CA) == 0.0E+0):
                return
        if (INCX == 1 and INCY == 1):
                "omp parallel for"
                for I in range(N):
                        CY[I] = CY[I] + CA*CX[I]
#
#        code for both increments equal to 1
#
        else:
#
#        code for unequal increments or equal increments
#          not equal to 1
#
                IX = 0
                IY = 0
                if (INCX < 0):
                        IX = (-N+1)*INCX
                if (INCY < 0):
                        IY = (-N+1)*INCY
                for I in range(N):
                        CY[IY] = CY[IY] + CA*CX[IX]
                        IX = IX + INCX
                        IY = IY + INCY
        return CY



import numpy as np
import builtins

#pythran export _create_binned_data(int64[:], int[:], float[:,:], int)
#pythran export _create_binned_data(int64[:], int[:], int[:,:], int)
#pythran export _create_binned_data(intc[:], int[:], float[:,:], int)
#pythran export _create_binned_data(intc[:], int[:], int[:,:], int)
#pythran export _create_binned_data(int[:], int[:], float[:,:], int)
#pythran export _create_binned_data(int[:], int[:], int[:,:], int)
def _create_binned_data(bin_numbers, unique_bin_numbers, values, vv):
    """ Create hashmap of bin ids to values in bins
    key: bin number
    value: list of binned data
    """
    bin_map = dict()
    for i in unique_bin_numbers:
        bin_map[i] = []
    for i in builtins.range(len(bin_numbers)):
        bin_map[bin_numbers[i]].append(values[vv, i])
    return bin_map


#pythran export _calc_binned_statistic(int, int64[:], float[:,:], float[:,:], str)
#pythran export _calc_binned_statistic(int, int64[:], float[:,:], int[:,:], str)
#pythran export _calc_binned_statistic(int, intc[:], float[:,:], float[:,:], str)
#pythran export _calc_binned_statistic(int, intc[:], float[:,:], int[:,:], str)
#pythran export _calc_binned_statistic(int, int[:], float[:,:], float[:,:], str)
#pythran export _calc_binned_statistic(int, int[:], float[:,:], int[:,:], str)
def _calc_binned_statistic(Vdim, bin_numbers, result, values, stat_func):
    unique_bin_numbers = np.unique(bin_numbers)
    for vv in builtins.range(Vdim):
        bin_map = _create_binned_data(bin_numbers, unique_bin_numbers,
                                      values, vv)
        for i in unique_bin_numbers:
            # if the stat_func is np.std, calc std only when binned data is 2
            # or more for speed up.
            if stat_func == 'std':
                if len(bin_map) >= 2:
                    result[vv, i] = np.std(np.array(bin_map[i]))
            elif stat_func == 'median':
                result[vv, i] = np.median(np.array(bin_map[i]))
            elif stat_func == 'min':
                result[vv, i] = np.min(np.array(bin_map[i]))
            elif stat_func == 'max':
                result[vv, i] = np.max(np.array(bin_map[i]))
            else:
                raise Exception('Exception: {stat_func} is not supported')


#from http://rosettacode.org/wiki/Primality_by_trial_division#Python
#pythran export test()
#runas test()
#FIXME unittest.skip

def prime(a):
    return not (a < 2 or any(a % x == 0 for x in range(2, int(a**0.5) + 1)))

def prime2(a):
    if a == 2: return True
    if a < 2 or a % 2 == 0: return False
    return not any(a % x == 0 for x in range(3, int(a**0.5) + 1, 2))

def prime3(a):
    if a < 2: return False
    if a == 2 or a == 3: return True # manually test 2 and 3
    if a % 2 == 0 or a % 3 == 0: return False # exclude multiples of 2 and 3

    maxDivisor = a**0.5
    d, i = 5, 2
    while d <= maxDivisor:
        if a % d == 0: return False
        d += i
        i = 6 - i # this modifies 2 into 4 and viceversa

    return True

def test():
    return [i for i in range(40) if prime(i)], [i for i in range(40) if prime2(i)], [i for i in range(40) if prime3(i)]



""" Checking if a number is a perfect square. """
# pythran export is_square(int)
# runas is_square(12** 5)
# from http://stackoverflow.com/questions/2489435/\
# how-could-i-check-if-a-number-is-a-perfect-square


def is_square(a_positive_int):
    """ Check if it is a perfect square. """
    x = a_positive_int // 2
    seen = {x}
    while x * x != a_positive_int:
        x = (x + (a_positive_int // x)) // 2
        if x in seen:
            return False
        seen.add(x)
    return True



import omp
def omp_task_firstprivate():
    sum = 1234
    result = 0
    LOOPCOUNT = 1000
    NUM_TASKS = 25

    known_sum = 1234 + (LOOPCOUNT * (LOOPCOUNT + 1)) / 2

    if 'omp parallel':
        in_parallel = omp.in_parallel()
        if 'omp single':
            for i in range(NUM_TASKS):
                if 'omp task firstprivate(sum)':
                    for j in range(LOOPCOUNT + 1):
                        'omp flush'
                        sum += j
                    if sum != known_sum:
                        if 'omp critical':
                            result += 1

    return result == 0 or not in_parallel



#pythran export run(float, float, float, float, float, float, int, int, float list list)
#bench run(0.,0.,90.,90., 1., 100., 80, 80, [ [i/10., i/10., i/20.] for i in range(160) ])
#runas run(0.,0.,90.,90., 1., 100., 80, 80, [ [i/10., i/10., i/20.] for i in range(80) ])
import math
def run(xmin, ymin, xmax, ymax, step, range_, range_x, range_y, t):
    pt = [ [0]*range_y for _ in range(range_x)]
    "omp parallel for"
    for i in range(range_x):
        for j in range(range_y):
            s = 0
            for k in t:
                tmp = 6368.* math.acos( math.cos(xmin+step*i)*math.cos( k[0] ) *
                                       math.cos((ymin+step*j)-k[1])+  math.sin(xmin+step*i)*math.sin(k[0]))
                if tmp < range_:
                    s+=k[2] / (1+tmp)
            pt[i][j] = s
    return pt



#from http://rosettacode.org/wiki/Range_extraction#Python
#pythran export test_range_extract(int list list)
#runas test_range_extract([[-8, -7, -6, -3, -2, -1, 0, 1, 3, 4, 5, 7, 8, 9, 10, 11, 14, 15, 17, 18, 19, 20], [0, 1, 2, 4, 6, 7, 8, 11, 12, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 27, 28, 29, 30, 31, 32, 33, 35, 36, 37, 38, 39]])

def range_extract(lst):
    'Yield 2-tuple ranges or 1-tuple single elements from list of increasing ints'
    lenlst = len(lst)
    i = 0
    while i< lenlst:
        low = lst[i]
        while i <lenlst-1 and lst[i]+1 == lst[i+1]: i +=1
        hi = lst[i]
        if   hi - low >= 2:
            yield [low, hi]
        elif hi - low == 1:
            yield [low]
            yield [hi]
        else:
            yield [low]
        i += 1

def test_range_extract(on):
    return [list(range_extract(i)) for i in on]



import math

def some_work():
    sum = 0;
    for i in range(0, 1000):
        sum += math.sqrt (i)

def omp_parallel_for_private():
    sum = 0
    i2 = 0

    'omp parallel for reduction(+: sum) schedule(static, 1) private(i2)'
    for i in range(1, 1001):
        i2 = i
        'omp flush'
        some_work()
        'omp flush'
        sum += i2

    known_sum = (1000 * (1000 + 1)) / 2
    return known_sum == sum;



# from the paper `using cython to speedup numerical python programs'
#pythran export timeloop(float, float, float, float, float, float list list, float list list, float list list)
#pythran export timeloop(float, float, float, float, float, int list list, int list list, int list list)
#bench A=[list(range(70)) for i in range(100)] ; B=[list(range(70)) for i in range(100)] ; C=[list(range(70)) for i in range(100)] ; timeloop(1.,2.,.01,.1,.18, A,B,C )
#runas A=[list(range(10)) for i in range(5)] ; B=[list(range(10)) for i in range(5)] ; C=[list(range(10)) for i in range(5)] ; timeloop(1.,2.,.1,.1,.2, A,B,C )
def timeloop(t, t_stop, dt, dx, dy, u, um, k):
    while t <= t_stop:
        t += dt
        new_u = calculate_u(dt, dx, dy, u, um, k)
        um = u
        u = new_u
    return u

def calculate_u(dt, dx, dy, u, um, k):
    up = [ [0.]*len(u[0]) for i in range(len(u)) ]
    "omp parallel for"
    for i in range(1, len(u)-1):
        for j in range(1, len(u[0])-1):
            up[i][j] = 2*u[i][j] - um[i][j] + \
                (dt/dx)**2*(
                        (0.5*(k[i+1][j] + k[i][j])*(u[i+1][j] - u[i][j]) -
                            0.5*(k[i][j] + k[i-1][j])*(u[i][j] - u[i-1][j]))) + \
                        (dt/dy)**2*(
                                (0.5*(k[i][j+1] + k[i][j])*(u[i][j+1] - u[i][j]) -
                                    0.5*(k[i][j] + k[i][j-1])*(u[i][j] - u[i][j-1])))
    return up



from numpy import pi

# pythran export unwrap_1d(float64[:], float64[:])


def unwrap_1d(image, unwrapped_image):
    '''Phase unwrapping using the naive approach.'''
    unwrapped_image[0] = image[0]
    periods = 0
    for i in range(1, image.shape[0]):
        difference = image[i] - image[i - 1]
        if difference > pi:
            periods -= 1
        elif difference < -pi:
            periods += 1
        unwrapped_image[i] = image[i] + 2 * pi * periods



from distutils.core import setup
from Cython.Build import cythonize

setup(
    name = "add",
    ext_modules = cythonize('add.pyx'),
    script_name = 'setup.py',
    script_args = ['build_ext', '--inplace']
)
import add
import numpy as np
print(add.add(np.ones((3,3)), np.ones((3,3))))




#pythran export CDOTU(int, complex list, int, complex list, int)
#runas CDOTU(2,[complex(1,2),complex(2,3),complex(3,4),complex(5,6)],2,[complex(3,4),complex(1,2),complex(2,3),complex(5,6)],3)
#bench sz = 20000000; in1 = map(complex, range(sz), range(sz)); in2 = map(complex, range(sz), range(sz));CDOTU(sz / 6,in1,2,in2,3)
def CDOTU(N,CX,INCX,CY,INCY):
#     .. Scalar Arguments ..
#     INTEGER INCX,INCY,N
#     ..
#     .. Array Arguments ..
#     COMPLEX CX(#),CY(#)
#     ..
#
#  Purpose
#  =======
#
#     CDOTU forms the dot product of two vectors.
#
#  Further Details
#  ===============
#
#     jack dongarra, linpack, 3/11/78.
#     modified 12/3/93, array(1) declarations changed to array(#)
#
#  =====================================================================
#
#     .. Local Scalars ..
#     COMPLEX CTEMP
#     INTEGER I,IX,IY
#     ..
    CTEMP = complex(0.0, 0.0)
    CDOTU = complex(0.0, 0.0)
    if (N <= 0):
        return
    if (INCX == 1 and INCY == 1):
#
#        code for both increments equal to 1
#
        for I in range(N):
            CTEMP = CTEMP + CX[I] * CY[I]
    else:
#
#        code for unequal increments or equal increments
#          not equal to 1
#
        IX = 0
        IY = 0
        if (INCX < 0):
            IX = (-N + 1) * INCX
        if (INCY < 0):
            IY = (-N + 1) * INCY
        for I in range(N):
            CTEMP = CTEMP + CX[IX] * CY[IY]
            IX = IX + INCX
            IY = IY + INCY
    return CTEMP



"""Color Mixer

NumPy does not do overflow checking when adding or multiplying
integers, so currently the only way to clip results efficiently
(without making copies of the data) is with an extension such as this
one.

"""

import numpy as np

# pythran export add(uint8[:,:,:], uint8[:,:,:], int, int)
# pythran export multiply(uint8[:,:,:], uint8[:,:,:], int, float)
# pythran export brightness(uint8[:,:,:], uint8[:,:,:], float, int)
# pythran export sigmoid_gamma(uint8[:,:,:], uint8[:,:,:], float, float)
# pythran export gamma(uint8[:,:,:], uint8[:,:,:], float)
# pythran export py_hsv_2_rgb(float, float, float)
# pythran export py_rgb_2_hsv(int, int, int)
# pythran export hsv_add (uint8[:,:,:], uint8[:,:,:], float, float, float)


def add(img, stateimg, channel, amount):
    """Add a given amount to a color channel of `stateimg`, and
    store the result in `img`.  Overflow is clipped.

    Parameters
    ----------
    img : (M, N, 3) ndarray of uint8
        Output image.
    stateimg : (M, N, 3) ndarray of uint8
        Input image.
    channel : int
        Channel (0 for "red", 1 for "green", 2 for "blue").
    amount : int
        Value to add.

    """
    height, width, _ = img.shape
    k = channel
    n = amount

    lut = [np.uint8()] * 256

    for l in range(256):
        op_result = l + n
        if op_result > 255:
            op_result = 255
        elif op_result < 0:
            op_result = 0

        lut[l] = np.uint8(op_result)

    for i in range(height):
        for j in range(width):
            img[i, j, k] = lut[stateimg[i, j, k]]


def multiply(img, stateimg, channel, amount):
    """Multiply a color channel of `stateimg` by a certain amount, and
    store the result in `img`.  Overflow is clipped.

    Parameters
    ----------
    img : (M, N, 3) ndarray of uint8
        Output image.
    stateimg : (M, N, 3) ndarray of uint8
        Input image.
    channel : int
        Channel (0 for "red", 1 for "green", 2 for "blue").
    amount : float
        Multiplication factor.

    """
    height, width, _ = img.shape
    k = channel
    n = amount

    lut = [np.uint8()] * 256

    for l in range(256):
        op_result = l * n
        if op_result > 255:
            op_result = 255
        elif op_result < 0:
            op_result = 0

        lut[l] = np.uint8(op_result)

    for i in range(height):
        for j in range(width):
            img[i, j, k] = lut[stateimg[i, j, k]]


def brightness(img, stateimg, factor, offset):
    """Modify the brightness of an image.
    'factor' is multiplied to all channels, which are
    then added by 'amount'. Overflow is clipped.

    Parameters
    ----------
    img : (M, N, 3) ndarray of uint8
        Output image.
    stateimg : (M, N, 3) ndarray of uint8
        Input image.
    factor : float
        Multiplication factor.
    offset : int
        Ammount to add to each channel.

    """

    height, width, _ = img.shape

    lut = [np.uint8()] * 256

    for k in range(256):
        op_result = k * factor + offset
        if op_result > 255:
            op_result = 255
        elif op_result < 0:
            op_result = 0

        lut[k] = np.uint8(op_result)

    for i in range(height):
        for j in range(width):
            img[i, j, 0] = lut[stateimg[i, j, 0]]
            img[i, j, 1] = lut[stateimg[i, j, 1]]
            img[i, j, 2] = lut[stateimg[i, j, 2]]


def sigmoid_gamma(img,
                  stateimg,
                  alpha, beta):

    height, width, _ = img.shape

    c1 = 1 / (1 + np.exp(beta))
    c2 = 1 / (1 + np.exp(beta - alpha)) - c1

    lut = [np.uint8()] * 256

    # compute the lut
    for k in range(256):
        lut[k] = np.uint8(((1 / (1 + np.exp(beta - (k / 255.) * alpha)))
                           - c1) * 255 / c2)
    for i in range(height):
        for j in range(width):
            img[i, j, 0] = lut[stateimg[i, j, 0]]
            img[i, j, 1] = lut[stateimg[i, j, 1]]
            img[i, j, 2] = lut[stateimg[i, j, 2]]


def gamma(img,
          stateimg,
          gamma):

    height, width, _ = img.shape

    lut = [np.uint8()] * 256

    if gamma == 0:
        gamma = 0.00000000000000000001
    gamma = 1./gamma

    # compute the lut
    for k in range(256):
        lut[k] = np.uint8((pow((k / 255.), gamma) * 255))

    for i in range(height):
        for j in range(width):
            img[i, j, 0] = lut[stateimg[i, j, 0]]
            img[i, j, 1] = lut[stateimg[i, j, 1]]
            img[i, j, 2] = lut[stateimg[i, j, 2]]


def rgb_2_hsv(RGB, HSV):
    R, G, B = RGB

    if R > 255:
        R = 255
    elif R < 0:
        R = 0

    if G > 255:
        G = 255
    elif G < 0:
        G = 0

    if B > 255:
        B = 255
    elif B < 0:
        B = 0

    if R < G:
        MIN = R
        MAX = G
    else:
        MIN = G
        MAX = R

    if B < MIN:
        MIN = B
    elif B > MAX:
        MAX = B
    else:
        pass

    V = MAX / 255.

    if MAX == MIN:
        H = 0.
    elif MAX == R:
        H = (60 * (G - B) / (MAX - MIN) + 360) % 360
    elif MAX == G:
        H = 60 * (B - R) / (MAX - MIN) + 120
    else:
        H = 60 * (R - G) / (MAX - MIN) + 240

    if MAX == 0:
        S = 0
    else:
        S = 1 - MIN / MAX

    HSV[0] = H
    HSV[1] = S
    HSV[2] = V


def hsv_2_rgb(HSV, RGB):

    H, S, V = HSV

    if H > 360:
        H = H % 360
    elif H < 0:
        H = 360 - ((-1 * H) % 360)
    else:
        pass

    if S > 1:
        S = 1
    elif S < 0:
        S = 0
    else:
        pass

    if V > 1:
        V = 1
    elif V < 0:
        V = 0
    else:
        pass

    hi = int(H / 60.) % 6
    f = (H / 60.) - int(H / 60.)
    p = V * (1 - S)
    q = V * (1 - f * S)
    t = V * (1 - (1 - f) * S)

    if hi == 0:
        r = V
        g = t
        b = p
    elif hi == 1:
        r = q
        g = V
        b = p
    elif hi == 2:
        r = p
        g = V
        b = t
    elif hi == 3:
        r = p
        g = q
        b = V
    elif hi == 4:
        r = t
        g = p
        b = V
    else:
        r = V
        g = p
        b = q

    RGB[0] = r
    RGB[1] = g
    RGB[2] = b


def py_hsv_2_rgb(H, S, V):
    '''Convert an HSV value to RGB.

    Automatic clipping.

    Parameters
    ----------
    H : float
        From 0. - 360.
    S : float
        From 0. - 1.
    V : float
        From 0. - 1.

    Returns
    -------
    out : (R, G, B) ints
        Each from 0 - 255

    conversion convention from here:
    http://en.wikipedia.org/wiki/HSL_and_HSV

    '''

    HSV = [H, S, V]
    RGB = [0] * 3

    hsv_2_rgb(HSV, RGB)

    R = int(RGB[0] * 255)
    G = int(RGB[1] * 255)
    B = int(RGB[2] * 255)

    return R, G, B


def py_rgb_2_hsv(R, G, B):
    '''Convert an HSV value to RGB.

    Automatic clipping.

    Parameters
    ----------
    R : int
        From 0. - 255.
    G : int
        From 0. - 255.
    B : int
        From 0. - 255.

    Returns
    -------
    out : (H, S, V) floats
        Ranges (0...360), (0...1), (0...1)

    conversion convention from here:
    http://en.wikipedia.org/wiki/HSL_and_HSV

    '''
    RGB = [float(R), float(G), float(B)]
    HSV = [0.] * 3

    rgb_2_hsv(RGB, HSV)

    return HSV


def hsv_add(img, stateimg, h_amt, s_amt, v_amt):
    """Modify the image color by specifying additive HSV Values.

    Since the underlying images are RGB, all three values HSV
    must be specified at the same time.

    The RGB triplet in the image is converted to HSV, the operation
    is applied, and then the HSV triplet is converted back to RGB

    HSV values are scaled to H(0. - 360.), S(0. - 1.), V(0. - 1.)
    then the operation is performed and any overflow is clipped, then the
    reverse transform is performed. Those are the ranges to keep in mind,
    when passing in values.

    Parameters
    ----------
    img : (M, N, 3) ndarray of uint8
        Output image.
    stateimg : (M, N, 3) ndarray of uint8
        Input image.
    h_amt : float
        Ammount to add to H channel.
    s_amt : float
        Ammount to add to S channel.
    v_amt : float
        Ammount to add to V channel.


    """

    height, width, _ = img.shape

    HSV = [0.] * 3
    RGB = [0.] * 3

    for i in range(height):
        for j in range(width):
            RGB[0] = stateimg[i, j, 0]
            RGB[1] = stateimg[i, j, 1]
            RGB[2] = stateimg[i, j, 2]

            rgb_2_hsv(RGB, HSV)

            # Add operation
            HSV[0] += h_amt
            HSV[1] += s_amt
            HSV[2] += v_amt

            hsv_2_rgb(HSV, RGB)

            RGB[0] *= 255
            RGB[1] *= 255
            RGB[2] *= 255

            img[i, j, 0] = RGB[0]
            img[i, j, 1] = RGB[1]
            img[i, j, 2] = RGB[2]


# pythran export hsv_multiply(uint8[:,:,:], uint8[:,:,:], float, float, float)
def hsv_multiply(img,
                 stateimg,
                 h_amt, s_amt, v_amt):
    """Modify the image color by specifying multiplicative HSV Values.

    Since the underlying images are RGB, all three values HSV
    must be specified at the same time.

    The RGB triplet in the image is converted to HSV, the operation
    is applied, and then the HSV triplet is converted back to RGB

    HSV values are scaled to H(0. - 360.), S(0. - 1.), V(0. - 1.)
    then the operation is performed and any overflow is clipped, then the
    reverse transform is performed. Those are the ranges to keep in mind,
    when passing in values.

    Note that since hue is in degrees, it makes no sense to multiply
    that channel, thus an add operation is performed on the hue. And the
    values given for h_amt, should be the same as for hsv_add

    Parameters
    ----------
    img : (M, N, 3) ndarray of uint8
        Output image.
    stateimg : (M, N, 3) ndarray of uint8
        Input image.
    h_amt : float
        Ammount to add to H channel.
    s_amt : float
        Ammount by which to multiply S channel.
    v_amt : float
        Ammount by which to multiply V channel.


    """

    height, width, _ = img.shape

    HSV = [0.] * 3
    RGB = [0.] * 3

    for i in range(height):
        for j in range(width):
            RGB[0] = stateimg[i, j, 0]
            RGB[1] = stateimg[i, j, 1]
            RGB[2] = stateimg[i, j, 2]

            rgb_2_hsv(RGB, HSV)

            # Multiply operation
            HSV[0] += h_amt
            HSV[1] *= s_amt
            HSV[2] *= v_amt

            hsv_2_rgb(HSV, RGB)

            RGB[0] *= 255
            RGB[1] *= 255
            RGB[2] *= 255

            img[i, j, 0] = RGB[0]
            img[i, j, 1] = RGB[1]
            img[i, j, 2] = RGB[2]



#from http://stackoverflow.com/questions/17112550/python-and-numba-for-vectorized-functions
#pythran export calculate_vibr_energy(float[], float[],int [])
#pythran export calculate_vibr_energy(float[], float[], int)
#pythran export calculate_vibr_energy(float[], float[], float)
#pythran export calculate_vibr_energy(float[], float[], float [])
#runas import numpy as np ; a = np.sin(np.ones(1000000)) ; b = np.cos(np.ones(1000000)) ; n = np.arange(1000000); calculate_vibr_energy(a, b, n)
#runas import numpy as np ; a = np.sin(np.ones(1000000)) ; b = np.cos(np.ones(1000000)) ; calculate_vibr_energy(a, b, 10)
#runas import numpy as np ; a = np.sin(np.ones(1000000)) ; b = np.cos(np.ones(1000000)) ; calculate_vibr_energy(a, b, 10.)
#runas import numpy as np ; a = np.sin(np.ones(1000000)) ; b = np.cos(np.ones(1000000)) ; n = np.arange(1000000, dtype=np.double); calculate_vibr_energy(a, b, n)
#bench import numpy as np ; a = np.sin(np.ones(1000000)) ; b = np.cos(np.ones(1000000)) ; n = np.arange(1000000); calculate_vibr_energy(a, b, n)
import numpy
def calculate_vibr_energy(harmonic, anharmonic, i):
    return numpy.exp(-harmonic * i - anharmonic * (i ** 2))



#pythran export solve(int)
#unittest.skip requires extensive typing, use enable_two_steps_typing from the cfg file 
def hits(x1, y1, x2, y2):
    "Check whether a queen positioned at (x1, y1) will hit a queen at position (x2, y2)"
    return x1 == x2 or y1 == y2 or abs(x1 - x2) == abs(y1 - y2)

def hitsany(x, y, queens_x, queens_y):
    "Check whether a queen positioned at (x1, y1) will hit any other queen"
    for i in range(len(queens_x)):
        if hits(x, y, queens_x[i], queens_y[i]):
            return True

    return False

def _solve(n, queens_x, queens_y):
    "Solve the queens puzzle"
    if n == 0:
        return True

    for x in range(1, 9):
        for y in range(1, 9):
            if not hitsany(x, y, queens_x, queens_y):
                queens_x.append(x)
                queens_y.append(y)

                if _solve(n - 1, queens_x, queens_y):
                    return True

                queens_x.pop()
                queens_y.pop()

    return False

def solve(n):
    queens_x = []
    queens_y = []

    if _solve(n, queens_x, queens_y):
        return queens_x, queens_y
    else:
        return None


#print(solve(8))
# %timeit solve(8)

# Comment out @jit/@autojit
# print(solve(8))
# %timeit solve(8)




import numpy as np
def csum(data):
    result = 0.
    #omp parallel for reduction(+: result)
    for d in data:
        result += d
    return result

def declare_reduction_complex_implicit():
    data = [-1j, 2j, 3j, 1j, 4j, 5j, 6j, -7j]
    res = csum(data)
    return res



#from:  https://stackoverflow.com/questions/50658884/why-this-numba-code-is-6x-slower-than-numpy-code
#runas import numpy as np; x1 = np.array([[1.,2,3]]) ; x2 = np.array([[1.,2,3],[2,3,4], [3,4,5], [4,5,6], [5,6,7]]); euclidean_distance_square(x1, x2)
#pythran export euclidean_distance_square(float64[1,:], float64[:,:])
import numpy as np
def euclidean_distance_square(x1, x2):
    return -2*np.dot(x1, x2.T) + np.sum(np.square(x1), axis=1)[:, np.newaxis] + np.sum(np.square(x2), axis=1)



#pythran export test(int)
#runas test(12)
#bench test(33)
def rfibo(n):
    if n < 2: return n
    else:
        n_1 = rfibo(n-1)
        n_2 = rfibo(n-2)
        return n_1 + n_2
def fibo(n):
    if n < 10: return rfibo(n)
    else:
        n_1 = 0
        "omp task shared(n,n_1)"
        n_1 = fibo(n-1)
        n_2 = fibo(n-2)
        "omp taskwait"
        return n_1 + n_2

def test(n):
    "omp parallel"
    "omp single"
    f = fibo(n)
    return f



import unittest

unittest.skip
#final semble ne pas fonctionner
def omp_task_final():
    import omp
    from time import sleep
    error = 0
    NUM_TASKS = 25
    tids = list(range(NUM_TASKS))

    if 'omp parallel':
        if 'omp single':
            for i in range(NUM_TASKS):
                myi = i
                if 'omp task final(i>=10) private(k) firstprivate(myi)':
                    sleep(0.01)
                    tids[myi] = omp.get_thread_num()
            'omp taskwait'

    for i in range(10, NUM_TASKS):
        if tids[10] != tids[i]:
            print(i, tids[10], tids[i])
            error += 1

    print(error)
    return error == 0



#pythran export pselect(int)
#runas pselect(0)
#runas pselect(1)
def pselect(n):
    l=list()
    for k in (n, not n):
        if k:
            a=sel0
        else:
            a=sel1
        a(l)
    return l

def sel0(n):
    n.append(1)
def sel1(n):
    n.append(2.)



#runas import numpy as np; x = np.array([[1,0],[0, 1]], dtype=np.intp); ising_step(x)
#pythran export ising_step(intp[:,:])

import numpy as np
def ising_step(field, beta=0.4):
    N, M = field.shape
    for n_offset in range(2):
        for m_offset in range(2):
            for n in range(n_offset, N, 2):
                for m in range(m_offset, M, 2):
                    _ising_update(field, n, m, np.float32(beta))
    return field

def clamp(v, n):
    if v < 0: return v + n
    elif v < n: return v
    else: return v - n

def _ising_update(field, n, m, beta):
    total = 0
    N, M = field.shape
    for i in range(n-1, n+2):
        for j in range(m-1, m+2):
            if i == n and j == m:
                continue
            total += field[clamp(i, N), clamp(j, M)]
    dE = 2 * field[n, m] * total
    if dE <= 0:
        field[n, m] *= -1
    elif np.exp(-dE * beta) > 0.5: #np.random.rand():
        field[n, m] *= -1



#pythran export permutations(int list)
#runas permutations([1,4,5,6,12])
#bench in_ = range(9); permutations(in_)
def permutations(iterable):
    """permutations(range(3), 2) --> (0,1) (0,2) (1,0) (1,2) (2,0) (2,1)"""
    out=[]
    pool = tuple(iterable)
    n = len(pool)
    r = n
    indices = list(range(n))
    cycles = list(range(n-r+1, n+1))[::-1]
    out.append( tuple([pool[i] for i in indices[:r]]))
    while 1:
        for i in reversed(range(r)):
            cycles[i] -= 1
            if cycles[i] == 0:
                indices[i:] = indices[i+1:] + indices[i:i+1]
                cycles[i] = n - i
            else:
                j = cycles[i]
                indices[i], indices[-j] = indices[-j], indices[i]
                out.append( tuple([pool[i] for i in indices[:r]]))
                break
        else:
            return out



def aa(a):
    ret = []
    while a > 2:
        ret.append(0)
        a -=1
    return ret

def hh(value):
    ret = []
    while value > 0:
        a, b = _div_tuple(value, 10)
        ret.insert(0, 0)
        value = a
    return ret


def _div_tuple(base, div):
    a = base // div
    b = base % div
    return a, b



def omp_task():
    import omp
    from time import sleep
    NUM_TASKS = 25
    tids = list(range(NUM_TASKS))

    if 'omp parallel':
        use_parallel = omp.in_parallel()
        for i in range(NUM_TASKS):
            myi = i
            if 'omp task':
                sleep(0.01)
                tids[myi] = omp.get_thread_num()
    for i in range(NUM_TASKS):
        if tids[0] != tids[i]:
            return True

    return not use_parallel



# Author: Pim Schellart
# 2010 - 2011

"""Tools for spectral analysis of unequally sampled signals."""

import numpy as np

#pythran export _lombscargle(float64[], float64[], float64[])
##runas import numpy; x = numpy.arange(2., 12.); y = numpy.arange(1., 11.); z = numpy.arange(3., 13.); _lombscargle(x, y, z)

def _lombscargle(x, y, freqs):
    """
    _lombscargle(x, y, freqs)

    Computes the Lomb-Scargle periodogram.

    Parameters
    ----------
    x : array_like
        Sample times.
    y : array_like
        Measurement values (must be registered so the mean is zero).
    freqs : array_like
        Angular frequencies for output periodogram.

    Returns
    -------
    pgram : array_like
        Lomb-Scargle periodogram.

    Raises
    ------
    ValueError
        If the input arrays `x` and `y` do not have the same shape.

    See also
    --------
    lombscargle

    """

    # Check input sizes
    if x.shape != y.shape:
        raise ValueError("Input arrays do not have the same size.")

    # Create empty array for output periodogram
    pgram = np.empty_like(freqs)

    c = np.empty_like(x)
    s = np.empty_like(x)

    for i in range(freqs.shape[0]):

        xc = 0.
        xs = 0.
        cc = 0.
        ss = 0.
        cs = 0.

        c[:] = np.cos(freqs[i] * x)
        s[:] = np.sin(freqs[i] * x)

        for j in range(x.shape[0]):
            xc += y[j] * c[j]
            xs += y[j] * s[j]
            cc += c[j] * c[j]
            ss += s[j] * s[j]
            cs += c[j] * s[j]

        if freqs[i] == 0:
            raise ZeroDivisionError()

        tau = np.arctan2(2 * cs, cc - ss) / (2 * freqs[i])
        c_tau = np.cos(freqs[i] * tau)
        s_tau = np.sin(freqs[i] * tau)
        c_tau2 = c_tau * c_tau
        s_tau2 = s_tau * s_tau
        cs_tau = 2 * c_tau * s_tau

        pgram[i] = 0.5 * (((c_tau * xc + s_tau * xs)**2 / \
            (c_tau2 * cc + cs_tau * cs + s_tau2 * ss)) + \
            ((c_tau * xs - s_tau * xc)**2 / \
            (c_tau2 * ss - cs_tau * cs + s_tau2 * cc)))

    return pgram



import doctest
import inspect
import os
import pytest
import sys
import unittest

import pythran
from pythran import transformations, analyses, optimizations, types, log


class TestDoctest(unittest.TestCase):

    """
    Enable automatic doctest integration to unittest.

    Every module in the pythran package is scanned for doctests
    and one test per module is created
    """

    @pytest.mark.skipif(sys.platform == "win32",
                        reason="We should create a file for windows.")
    def test_tutorial(self):
        failed, _ = doctest.testfile('../../docs/TUTORIAL.rst')
        self.assertEqual(failed, 0)

    @pytest.mark.skipif(sys.platform == "win32",
                        reason="We should create a file for windows.")
    def test_internal(self):
        tmpfile = self.adapt_rst('../../docs/INTERNAL.rst')
        failed, _ = doctest.testfile(tmpfile, False)
        self.assertEqual(failed, 0)
        os.remove(tmpfile)

    @pytest.mark.skipif(sys.platform == "win32",
                        reason="We should create a file for windows.")
    def test_cli(self):
        tmpfile = self.adapt_rst('../../docs/CLI.rst')
        failed, _ = doctest.testfile(tmpfile, False)
        self.assertEqual(failed, 0)
        os.remove(tmpfile)

    def adapt_rst(self, relative_path):
        """
        replace '$>' with '>>>' and execute theses command lines by creating a shell
        return the path of the new adapted tmp file
        """
        import re
        from tempfile import NamedTemporaryFile
        filepath = os.path.join(os.path.dirname(__file__), relative_path)
        with open(filepath) as rst_doc:
            # hidden doctest
            sp = re.sub(r'\.\.(\s+>>>)', r'\1', rst_doc.read())
            sp = re.sub(r'^([ ]+)$', r'\1<BLANKLINE>', sp, flags=re.MULTILINE)
            # harmonize argparse output after python 3.10
            if sys.version_info >= (3, 10):
                sp = sp.replace('optional arguments:', 'options:')

        # hack to support setuptools-generated pythran / pythran-config scripts
        for tool, sub in (('pythran-config', 'python -m pythran.config'),
                          ('pythran', 'python -m pythran.run'),
                          ('python', sys.executable)):
            sp = re.sub(r'(\$>.*?[^#])\b' + tool + r'\b([^-.].*)$',
                        r'\1' + sub + r'\2',
                        sp,
                        flags=re.MULTILINE)

        # convert shell doctest into python ones
        sp = re.sub(r'\$>(.*?)$',
                    r'>>> import subprocess ; res = subprocess.check_output("\1", shell=True).decode("ascii").strip() ; print(res, end="")',
                    sp,
                    flags=re.MULTILINE)
        f = NamedTemporaryFile("w", delete=False)
        f.write(sp)
        f.close()
        return f.name

def generic_test_package(self, mod):
    # disable logging during doctest
    log.logging.getLogger('pythran').setLevel(log.logging.ERROR)

    failed, _ = doctest.testmod(mod)
    self.assertEqual(failed, 0)

def add_module_doctest(base, module_name):
    module = getattr(base, module_name)
    if inspect.ismodule(module):
        setattr(TestDoctest, 'test_' + module_name,
            lambda self: generic_test_package(self, module))

# doctest does not goes through imported variables,
# so manage the tests manually here
for module in (pythran, transformations, analyses, optimizations, types):
    for submodule in dir(module):
        add_module_doctest(module, submodule)

if __name__ == '__main__':
    unittest.main()



import unittest
from pythran.tests import TestFromDir
import os

class TestGWebb(TestFromDir):

    path = os.path.join(os.path.dirname(__file__),"g webb")


TestGWebb.populate(TestGWebb)


if __name__ == '__main__':
    unittest.main()



#runas solve()
#pythran export solve()
def solve():
    '''
    Starting with 1 and spiralling anticlockwise in the following way, a square spiral with side length 7 is formed.
    
    37 36 35 34 33 32 31
    38 17 16 15 14 13 30
    39 18  5  4  3 12 29
    40 19  6  1  2 11 28
    41 20  7  8  9 10 27
    42 21 22 23 24 25 26
    43 44 45 46 47 48 49
    
    It is interesting to note that the odd squares lie along the bottom right diagonal, but what is more interesting is that 8 out of the 13 numbers lying along both diagonals are prime; that is, a ratio of 8/13 ~ 62%.
    
    If one complete new layer is wrapped around the spiral above, a square spiral with side length 9 will be formed. If this process is continued, what is the side length of the square spiral for which the ratio of primes along both diagonals first falls below 10%?
    '''

    prime_list = [2, 3, 5, 7, 11, 13, 17, 19, 23]   # Ensure that this is initialised with at least 1 prime
    prime_dict = dict.fromkeys(prime_list, 1)

    def _isprime(n):
        ''' Raw check to see if n is prime. Assumes that prime_list is already populated '''
        isprime = n >= 2 and 1 or 0
        for prime in prime_list:                    # Check for factors with all primes
            if prime * prime > n: break             # ... up to sqrt(n)
            if not n % prime:
                isprime = 0
                break
        if isprime: prime_dict[n] = 1               # Maintain a dictionary for fast lookup
        return isprime

    def _refresh(x):
        ''' Refreshes primes upto x '''
        lastn      = prime_list[-1]
        while lastn <= x:                           # Keep working until we've got up to x
            lastn = lastn + 1                       # Check the next number
            if _isprime(lastn):
                prime_list.append(lastn)            # Maintain a list for sequential access

    _refresh(50000)

    width, diagonal, base, primes = 1, 1, 1, 0
    while True:
        width = width + 2
        increment = width - 1
        for i in range(0, 4):
            diagonal = diagonal + increment
            if i < 3 and _isprime(diagonal): primes += 1
        base = base + 4
        if primes * 10 < base:
            return width
            break

